@@ -160,38 +160,22 @@ static void ft_draw_bitmap(
 		int		ppb
 		)
 {
-	unsigned char *dest;
-	int i, j, p, q, x_max = x + bitmap->width, y_max = y + bitmap->rows;
+	unsigned char *dest = mem + y * w + x, *src = bitmap->buffer;
+	int i, j;
 
-	if ( ppb==1 )			// 8 bits per pixel greyscale
+	for (j = 0; j < bitmap->rows; j++)
 	{
-		for ( j = y, q = 0; j < y_max; j++, q++ )
+		if (ppb == 1)		// 8 bits per pixel greyscale
 		{
-			dest = mem + j*w;
-			for ( i = x, p = 0; i < x_max; i+=ppb, p++ )
-			{
-				dest[i] |= bitmap->buffer[q * bitmap->pitch + p];
-			}
+//			memcpy(dest, src, bitmap->width);
+			for (i = 0; i < bitmap->width; i++) dest[i] |= src[i];
 		}
-	}
-	if ( ppb==8 )			// 1 bit per pixel mono
-	{
-		int sh;
-
-		for ( j = y, q = 0; j < y_max; j++, q++ )
+		else if (ppb == 8)	// 1 bit per pixel mono
 		{
-			dest = mem + j*w;
-			for ( i = x, p = 0; i < x_max; i+=ppb, p++ )
-			{
-				for ( sh=0; sh<8; sh++ )
-				{
-					if ( sh+p >= bitmap->width ) break;
-
-					if (bitmap->buffer[q * bitmap->pitch + p] & (1<<(7-sh)) )
-						dest[i+sh] |= 255;
-				}
-			}
-		}
+			for (i = 0; i < bitmap->width; i++) dest[i] |=
+				(((int)src[i >> 3] >> (~i & 7)) & 1) * 255;
+		}			
+		dest += w; src += bitmap->pitch;
 	}
 }
 
@@ -1087,7 +1071,7 @@ static gint paste_text_ok( GtkWidget *widget, GdkEvent *event, gpointer data )
 
 	read_font_controls(fp);
 	ft_render_text();
-	if ( mem_clipboard != NULL ) pressed_paste_centre( NULL, NULL );
+	if (mem_clipboard) pressed_paste(TRUE);
 
 	delete_text( widget, data );
 
@@ -1760,7 +1744,7 @@ static void font_entry_changed(GtkWidget *widget, gpointer user)	// A GUI entry
 #endif		// U_FREETYPE
 
 
-void pressed_mt_text( GtkMenuItem *menu_item, gpointer user_data )
+void pressed_mt_text()
 {
 #ifdef U_FREETYPE
 	int		i;
@@ -1839,8 +1823,8 @@ void pressed_mt_text( GtkMenuItem *menu_item, gpointer user_data )
 	gtk_scrolled_window_add_with_viewport( GTK_SCROLLED_WINDOW(scrolledwindow), mem->preview_area);
 	gtk_widget_show( mem->preview_area );
 
-	gtk_signal_connect (GTK_OBJECT (mem->preview_area), "expose_event",
-		(GtkSignalFunc) preview_expose_event, NULL);
+	gtk_signal_connect(GTK_OBJECT(mem->preview_area), "expose_event",
+		GTK_SIGNAL_FUNC(preview_expose_event), NULL);
 
 //	TOGGLES
 
@@ -1877,7 +1861,7 @@ void pressed_mt_text( GtkMenuItem *menu_item, gpointer user_data )
 		inifile_get_gboolean( "fontAntialias2", FALSE ) );
 
 	for ( i=0; i<TX_TOGGS; i++ )
-		gtk_signal_connect(GTK_OBJECT(mem->toggle[i]), "clicked",
+		gtk_signal_connect(GTK_OBJECT(mem->toggle[i]), "toggled",
 			GTK_SIGNAL_FUNC(font_entry_changed), (gpointer)mem);
 	for ( i=0; i<TX_SPINS; i++ )
 		if (mem->spin[i]) spin_connect(mem->spin[i],
