@@ -1,5 +1,5 @@
 /*	help.c
-	Copyright (C) 2004-2008 Mark Tyler
+	Copyright (C) 2004-2009 Mark Tyler
 
 	This file is part of mtPaint.
 
@@ -29,7 +29,7 @@ _("Credits"),
 };
 
 static char *help_page0[] = {
-_("mtPaint 3.21 - Copyright (C) 2004-2008 The Authors\n"),
+_("mtPaint 3.31 - Copyright (C) 2004-2009 The Authors\n"),
 _("See 'Credits' section for a list of the authors.\n"),
 _("mtPaint is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.\n"),
 _("mtPaint is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.\n"),
@@ -70,7 +70,6 @@ _("  Ctrl-P            Preferences"),
 _("  Ctrl-I            Information\n"),
 _("  Ctrl-Z            Undo last action"),
 _("  Ctrl-R            Redo an undone action\n"),
-_("  C                 Command Line Window"),
 _("  V                 View Window"),
 _("  L                 Layers Window\n"),
 _("  A                 Draw open arrow head when using the line tool (size set by flow setting)"),
@@ -100,12 +99,13 @@ _("  F5                Toggle Main Toolbar"),
 _("  F6                Toggle Tools Toolbar"),
 _("  F7                Toggle Settings Toolbar"),
 _("  F8                Toggle Palette"),
-_("  F9                Selection Tool\n"),
+_("  F9                Selection Tool"),
+_("  F12               Toggle Dock Area\n"),
 _("  Ctrl + F1 - F12   Save current clipboard to file 1-12"),
 _("  Shift + F1 - F12  Load clipboard from file 1-12\n"),
 _("  Ctrl + 1, 2, ... , 0  Set opacity to 10%, 20%, ... , 100% (main or keypad numbers)"),
-_("  Ctrl + + or =     Increase opacity by 1%"),
-_("  Ctrl + -          Decrease opacity by 1%\n"),
+_("  Ctrl + + or =     Increase opacity by 1"),
+_("  Ctrl + -          Decrease opacity by 1\n"),
 _("  Home              Show or hide main window menu/toolbar/status bar/palette"),
 _("  Page Up           Scale Image"),
 _("  Page Down         Resize Image canvas"),
@@ -120,6 +120,7 @@ _("  Ctrl+Left button     Choose colour A from under mouse pointer"),
 _("  Ctrl+Middle button   Create colour A/B and pattern based on the RGB colour in A (RGB images only)"),
 _("  Ctrl+Right button    Choose colour B from under mouse pointer"),
 _("  Ctrl+Scroll Wheel    Scroll the main edit window left or right\n"),
+_("  Ctrl+Double click    Set colour A or B to average colour under brush square or selection marquee (RGB only)\n"),
 _("  Shift+Right button   Selects the point which will be the centre of the image after the next zoom\n\n"),
 _("You can fixate the X/Y co-ordinates while moving the mouse:\n"),
 _("  Shift                Constrain mouse movements to vertical line"),
@@ -152,17 +153,20 @@ _("Vlastimil Krejcir"),
 _("William Kern\n\n"),
 _("Translations\n"),
 _("Brazilian Portuguese - Paulo Trevizan"),
-_("Czech - Pavel Ruzicka"),
-_("French - Nicolas Velin, Pascal Billard, Sylvain Cresto"),
+_("Czech - Pavel Ruzicka, Martin Petricek"),
+_("Dutch - Hans Strijards"),
+_("French - Nicolas Velin, Pascal Billard, Sylvain Cresto, Johan Serre, Plume"),
 _("Galician - Miguel Anxo Bouzada"),
 _("German - Oliver Frommel"),
+_("Italian - Angelo Gemmi"),
 _("Japanese - Norihiro YONEDA"),
 _("Polish - Simek, LucaS"),
 _("Portuguese - Israel G. Lugo, Tiago Silva"),
 _("Russian - Sergey Irupin, Dmitry Groshev"),
-_("Simplified Chinese - puppychinese(Cecc)"),
+_("Simplified Chinese - Cecc"),
 _("Slovak - Jozef Riha"),
-_("Spanish - Guadalinex Team (Junta de Andalucia), Antonio Sanchez Leon"),
+_("Spanish - Guadalinex Team (Junta de Andalucia), Antonio Sanchez Leon, Miguel Anxo Bouzada, Francisco Jose Rey"),
+_("Swedish - Daniel Nylander, Fredrik Forsmo"),
 _("Taiwanese Chinese - Wei-Lun Chao"),
 _("Turkish - Muhammet Kara, Tutku Dalmaz"),
 NULL };
