@@ -1,5 +1,5 @@
 /*	memory.h
-	Copyright (C) 2004-2008 Mark Tyler and Dmitry Groshev
+	Copyright (C) 2004-2009 Mark Tyler and Dmitry Groshev
 
 	This file is part of mtPaint.
 
@@ -28,13 +28,8 @@
 /* !!! If MAX_WIDTH * MAX_HEIGHT * max bpp won't fit into int, lots of code
  * !!! will have to be modified to use size_t instead */
 
-#ifdef U_GUADALINEX
-	#define DEFAULT_WIDTH 800
-	#define DEFAULT_HEIGHT 600
-#else
-	#define DEFAULT_WIDTH 640
-	#define DEFAULT_HEIGHT 480
-#endif
+#define DEFAULT_WIDTH 640
+#define DEFAULT_HEIGHT 480
 
 /* Palette area layout */
 
@@ -100,12 +95,10 @@
 
 typedef unsigned char *chanlist[NUM_CHANNELS];
 
-#define UNDO_TILEMAP_SIZE 32
-
 typedef struct {
 	chanlist img;
 	png_color *pal_;
-	unsigned char tilemap[UNDO_TILEMAP_SIZE], *tileptr;
+	unsigned char *tileptr;
 	int cols, width, height, bpp, flags;
 	size_t size;
 } undo_item;
@@ -116,6 +109,7 @@ typedef struct {
 	int done;		// Undo images that we have behind current image (i.e. possible UNDO)
 	int redo;		// Undo images that we have ahead of current image (i.e. possible REDO)
 	int max;		// Total number of undo slots
+	size_t size;		// Total used memory (0 means count it anew)
 } undo_stack;
 
 typedef struct {
@@ -124,7 +118,6 @@ typedef struct {
 	int cols;	// Number of colours in the palette: 1..256 or 0 for no image
 	int bpp;		// Bytes per pixel = 1 or 3
 	int width, height;	// Image geometry
-	size_t size;
 	undo_stack undo_;	// Image's undo stack
 } image_info;
 
@@ -156,8 +149,8 @@ typedef struct {
 	int len, rep, ofs;	// Gradient length, repeat, and offset
 	int gmode, rmode;	// Gradient mode and repeat mode
 	/* Derived values */
-	double wrep, wil1, wil2, xv, yv;
-	int wmode;
+	double wrep, wil1, wil2, xv, yv, wa;
+	int wmode, wrmode;
 } grad_info;
 
 typedef struct {
@@ -179,16 +172,21 @@ int grad_opacity;			// Preview opacity
 
 /* Gradient modes */
 #define GRAD_MODE_NONE     0
-#define GRAD_MODE_LINEAR   1
-#define GRAD_MODE_BILINEAR 2
-#define GRAD_MODE_RADIAL   3
-#define GRAD_MODE_SQUARE   4
+#define GRAD_MODE_BURST    1
+#define GRAD_MODE_LINEAR   2
+#define GRAD_MODE_BILINEAR 3
+#define GRAD_MODE_RADIAL   4
+#define GRAD_MODE_SQUARE   5
+#define GRAD_MODE_ANGULAR  6
+#define GRAD_MODE_CONICAL  7
 
 /* Boundary conditions */
 #define GRAD_BOUND_STOP    0
 #define GRAD_BOUND_LEVEL   1
 #define GRAD_BOUND_REPEAT  2
 #define GRAD_BOUND_MIRROR  3
+#define GRAD_BOUND_STOP_A  4 /* Stop mode for angular gradient */
+#define GRAD_BOUND_REP_A   5 /* Repeat mode for same */
 
 /* Gradient types */
 #define GRAD_TYPE_RGB      0
@@ -218,18 +216,39 @@ int mem_gradient;
 int blend_mode;
 
 /* Blend modes */
-#define BLEND_NORMAL 0
-#define BLEND_HUE    1
-#define BLEND_SAT    2
-#define BLEND_VALUE  3
-#define BLEND_COLOR  4
-#define BLEND_SATPP  5
+enum {
+	/* RGB-only modes */
+	BLEND_NORMAL = 0,
+	BLEND_HUE,
+	BLEND_SAT,
+	BLEND_VALUE,
+	BLEND_COLOR,
+	BLEND_SATPP,
+
+	/* Per-channel modes */
+	BLEND_MULT,
+	BLEND_DIV,
+	BLEND_SCREEN,
+// !!! No "Overlay" - it is a reverse "Hard Light"
+	BLEND_DODGE,
+	BLEND_BURN,
+	BLEND_HLIGHT,
+	BLEND_SLIGHT,
+	BLEND_DIFF,
+	BLEND_DARK,
+	BLEND_LIGHT,
+	BLEND_GRAINX,
+	BLEND_GRAINM,
+
+	BLEND_NMODES
+};
+#define BLEND_1BPP BLEND_MULT /* First one-byte mode */
 
-#define BLEND_NMODES 6
 #define BLEND_MMASK    0x7F
 #define BLEND_REVERSE  0x80
 #define BLEND_RGBSHIFT 8
 
+
 /// FLOOD FILL SETTINGS
 
 double flood_step;
@@ -237,6 +256,10 @@ int flood_cube, flood_img, flood_slide;
 
 int smudge_mode;
 
+/// QUANTIZATION SETTINGS
+
+int quan_sqrt;	// "Diameter based weighting" - use sqrt of pixel count
+
 /// IMAGE
 
 #define MIN_UNDO 11	// Number of undo levels + 1
@@ -308,17 +331,12 @@ int brush_tool_type;			// Last brush tool type
 int mem_brush_list[81][3];		// Preset brushes parameters
 int mem_nudge;				// Nudge pixels per SHIFT+Arrow key during selection/paste
 
-int mem_preview;			// Preview an RGB change
 int mem_prev_bcsp[6];			// BR, CO, SA, POSTERIZE, GAMMA, Hue
 
 int mem_undo_limit;		// Max MB memory allocation limit
+int mem_undo_common;		// Percent of undo space in common arena
 int mem_undo_opacity;		// Use previous image for opacity calculations?
 
-/// GRID
-
-int mem_show_grid, mem_grid_min;	// Boolean show toggle & minimum zoom to show it at
-unsigned char mem_grid_rgb[3];		// RGB colour of grid
-
 /// PATTERNS
 
 unsigned char mem_pattern[8 * 8];	// Current pattern
@@ -344,6 +362,24 @@ extern unsigned char mem_pals[];	// RGB screen memory holding current palette
 int mem_background;			// Non paintable area
 int mem_histogram[256];
 
+/// Floored integer division
+
+static inline int floor_div(int dd, int dr)
+{
+	return (dd / dr - (dd % dr < 0)); // optimizes to perfection on x86
+}
+
+/// Multiblock allocator
+
+void *multialloc(int align, void *ptr, int size, ...);
+
+/// Vectorized low-level drawing function
+
+void (*put_pixel)(int x, int y);
+
+	// Intersect outer & inner rectangle, write out what it separates into
+int clip4(int *xywh04, int xo, int yo, int wo, int ho, int xi, int yi, int wi, int hi);
+
 /// Line iterator
 
 /* Indices 0 and 1 are current X and Y, 2 is number of pixels remaining */
@@ -352,10 +388,11 @@ typedef int linedata[10];
 void line_init(linedata line, int x0, int y0, int x1, int y1);
 int line_step(linedata line);
 void line_nudge(linedata line, int x, int y);
+int line_clip(linedata line, int *vxy, int *step);
 
 /// Procedures
 
-void init_istate();		// Set initial state of image variables
+void init_istate(image_state *state, image_info *image);	// Set initial state of image variables
 int init_undo(undo_stack *ustack, int depth);	// Create new undo stack of a given depth
 void update_undo_depth();	// Resize all undo stacks
 
@@ -368,7 +405,7 @@ int mem_count_all_cols_real(unsigned char *im, int w, int h);	// Count all colou
 int mem_cols_used(int max_count);		// Count colours used in main RGB image
 int mem_cols_used_real(unsigned char *im, int w, int h, int max_count, int prog);
 			// Count colours used in RGB chunk and dump to found table
-void mem_cols_found_dl(unsigned char userpal[3][256]);		// Convert results ready for DL code
+void mem_cols_found(png_color *userpal);	// Convert colours list into palette
 
 
 int read_hex( char in );			// Convert character to hex value 0..15.  -1=error
@@ -380,9 +417,14 @@ int read_hex_dub( char *in );			// Read hex double
 
 //	Clear/remove image data
 void mem_free_image(image_info *image, int mode);
+
+#define AI_COPY   1 /* Duplicate source channels, not insert them */
+#define AI_NOINIT 2 /* Do not initialize source-less channels */
+#define AI_CLEAR  4 /* Initialize image structure first */
+
 //	Allocate new image data
-int mem_alloc_image(image_info *image, int w, int h, int bpp, int cmask,
-	chanlist src);
+int mem_alloc_image(int mode, image_info *image, int w, int h, int bpp,
+	int cmask, chanlist src);
 //	Allocate space for new image, removing old if needed
 int mem_new( int width, int height, int bpp, int cmask );
 //	Allocate new clipboard, removing or preserving old as needed
@@ -397,11 +439,26 @@ size_t mem_used();
 //	Return the number of bytes used in image + undo in all layers
 size_t mem_used_layers();
 
-void mem_bacteria( int val );			// Apply bacteria effect val times the canvas area
+#define FX_EDGE       0
+#define FX_EMBOSS     2
+#define FX_SHARPEN    3
+#define FX_SOFTEN     4
+#define FX_SOBEL      5
+#define FX_PREWITT    6
+#define FX_GRADIENT   7
+#define FX_ROBERTS    8
+#define FX_LAPLACE    9
+#define FX_KIRSCH    10
+#define FX_ERODE     11
+#define FX_DILATE    12
+#define FX_MORPHEDGE 13
+
 void do_effect( int type, int param );		// 0=edge detect 1=UNUSED 2=emboss
+void mem_bacteria( int val );			// Apply bacteria effect val times the canvas area
 void mem_gauss(double radiusX, double radiusY, int gcor);
 void mem_unsharp(double radius, double amount, int threshold, int gcor);
 void mem_dog(double radiusW, double radiusN, int norm, int gcor);
+void mem_kuwahara(int r, int gcor);
 
 
 /// PALETTE PROCS
@@ -411,17 +468,15 @@ void mem_pal_load_def();		// Load default palette
 
 #define mem_pal_copy(A, B) memcpy((A), (B), SIZEOF_PALETTE)
 void mem_pal_init();			// Initialise whole of palette RGB
-int mem_pal_cmp( png_color *pal1,	// Count itentical palette entries
-	png_color *pal2 );
 void mem_greyscale(int gcor);		// Convert image to greyscale
 int mem_convert_rgb();			// Convert image to RGB
-int mem_convert_indexed(int img);	// Convert image to Indexed Palette
+int mem_convert_indexed();		// Convert image to Indexed Palette
 //	Quantize image using Max-Min algorithm
 int maxminquan(unsigned char *inbuf, int width, int height, int quant_to,
-	unsigned char userpal[3][256]);
+	png_color *userpal);
 //	Quantize image using PNN algorithm
 int pnnquan(unsigned char *inbuf, int width, int height, int quant_to,
-	unsigned char userpal[3][256]);
+	png_color *userpal);
 //	Convert RGB->indexed using error diffusion with variety of options
 int mem_dither(unsigned char *old, int ncols, short *dither, int cspace,
 	int dist, int limit, int selc, int serpent, int rgb8b, double emult);
@@ -438,10 +493,8 @@ void mem_mask_setall(char val);		// Clear/set all masks
 void mem_mask_init();			// Initialise RGB protection mask
 int mem_protected_RGB(int intcol);	// Is this intcol in list?
 
-void mem_swap_cols();			// Swaps colours and update memory
-void repaint_swatch( int index );	// Update a palette swatch
+void mem_swap_cols(int redraw);		// Swaps colours and update memory
 void mem_get_histogram(int channel);	// Calculate how many of each colour index is on the canvas
-int do_posterize(int val, int posty);	// Posterize a number
 int scan_duplicates();			// Find duplicate palette colours
 void remove_duplicates();		// Remove duplicate palette colours - call AFTER scan_duplicates
 int mem_remove_unused_check();		// Check to see if we can remove unused palette colours
@@ -460,6 +513,7 @@ void set_zoom_centre( int x, int y );
 
 // Nonclassical HSV: H is 0..6, S is 0.. 1, V is 0..255
 void rgb2hsv(unsigned char *rgb, double *hsv);
+void hsv2rgb(unsigned char *rgb, double *hsv);
 
 //// UNDO
 
@@ -492,9 +546,12 @@ void update_undo(image_info *image);	// Copy image state into current undo frame
 //	Try to allocate a memory block, releasing undo frames if needed
 void *mem_try_malloc(size_t size);
 
-
 //// Drawing Primitives
 
+int sb_xywh[4];				// Backbuffer placement
+int init_sb();				// Create shapeburst backbuffer
+void render_sb();			// Render from shapeburst backbuffer
+
 int mem_clip_mask_init(unsigned char val);		// Initialise the clipboard mask
 //	Extract alpha info from RGB clipboard
 int mem_scale_alpha(unsigned char *img, unsigned char *alpha,
@@ -505,10 +562,10 @@ void mem_clip_mask_set(unsigned char val);		// Mask colours A and B on the clipb
 void mem_clip_mask_clear();				// Clear the clipboard mask
 
 void do_clone(int ox, int oy, int nx, int ny, int opacity, int mode);
-#define mem_smudge(A, B, C, D) do_clone((A), (B), (C), (D), MEM_BPP == 3 ? \
-	tool_opacity / 2 : 127, smudge_mode && mem_undo_opacity)
-#define mem_clone(A, B, C, D) do_clone((A), (B), (C), (D), MEM_BPP == 3 ? \
-	tool_opacity : -1, mem_undo_opacity)
+#define mem_smudge(A, B, C, D) do_clone((A), (B), (C), (D), tool_opacity / 2, \
+	smudge_mode && mem_undo_opacity)
+#define mem_clone(A, B, C, D) do_clone((A), (B), (C), (D), tool_opacity, \
+	mem_undo_opacity)
 
 //	Apply colour transform
 void do_transform(int start, int step, int cnt, unsigned char *mask,
@@ -537,6 +594,7 @@ void mem_threshold(unsigned char *img, int len, int level);	// Threshold channel
 void mem_demultiply(unsigned char *img, unsigned char *alpha, int len, int bpp);
 
 void set_xlate(unsigned char *xlat, int bpp);			// Build bitdepth translation table
+int is_filled(unsigned char *data, unsigned char val, int len);	// Check if byte array is all one value
 
 void flood_fill( int x, int y, unsigned int target );
 
@@ -560,8 +618,7 @@ void draw_quad(linedata line1, linedata line2, linedata line3, linedata line4);
 
 #define MEM_BPP (mem_channel == CHN_IMAGE ? mem_img_bpp : 1)
 #define BPP(x) ((x) == CHN_IMAGE ? mem_img_bpp : 1)
-
-#define GET_PIXEL(x,y) (mem_img[CHN_IMAGE][(x) + (y) * mem_width])
+#define IS_INDEXED ((mem_channel == CHN_IMAGE) && (mem_img_bpp == 1))
 
 #define POSTERIZE_MACRO res = 0.49 + ( ((1 << posty) - 1) * ((float) res)/255);\
 			res = 0.49 + 255 * ( ((float) res) / ((1 << posty) - 1) );
@@ -573,13 +630,17 @@ void process_mask(int start, int step, int cnt, unsigned char *mask,
 	unsigned char *trans, int opacity, int noalpha);
 void process_img(int start, int step, int cnt, unsigned char *mask,
 	unsigned char *imgr, unsigned char *img0, unsigned char *img,
-	int opacity, int sourcebpp);
+	int sourcebpp, int destbpp);
 void paste_pixels(int x, int y, int len, unsigned char *mask, unsigned char *img,
 	unsigned char *alpha, unsigned char *trans, int opacity);
+void copy_area(image_info *dest, image_info *src, int x, int y);
+
+// Retroactive masking - by blending with undo frame
+void mask_merge(unsigned char *old, int channel, unsigned char *mask);
 
 int pixel_protected(int x, int y);				// generic
 void row_protected(int x, int y, int len, unsigned char *mask);
-void put_pixel( int x, int y );					// generic
+void put_pixel_def( int x, int y );				// generic
 int get_pixel( int x, int y );					// generic
 int get_pixel_RGB( int x, int y );				// converter
 int get_pixel_img( int x, int y );				// from image
@@ -597,12 +658,13 @@ void prep_grad(int start, int step, int cnt, int x, int y, unsigned char *mask,
 #define GRAD_CUSTOM_OPAC(X) (GRAD_POINTS * ((X) * 4 + 4))
 #define GRAD_CUSTOM_OMAP(X) (GRAD_POINTS * ((X) * 4 + 5))
 
-void blend_channel(int start, int step, int cnt, unsigned char *mask,
-	unsigned char *dest, unsigned char *src, int opacity);
 void blend_indexed(int start, int step, int cnt, unsigned char *rgb,
 	unsigned char *img0, unsigned char *img,
 	unsigned char *alpha0, unsigned char *alpha, int opacity);
 
+int mem_skew(double xskew, double yskew, int type, int gcor);
+
+int average_pixels(unsigned char *rgb, int iw, int ih, int x, int y, int w, int h);
 
 #define IF_IN_RANGE( x, y ) if ( x>=0 && y>=0 && x<mem_width && y<mem_height )
 
@@ -618,3 +680,41 @@ void blend_indexed(int start, int step, int cnt, unsigned char *rgb,
 #else
 #define ALIGNTO(p,s) ((void *)(p))
 #endif
+
+/* x87 FPU uses long doubles internally, which may cause calculation results
+ * to depend on emitted assembly code, and change in mysterious ways depending
+ * on source structure and current optimizations.
+ * http://gcc.gnu.org/bugzilla/show_bug.cgi?id=323
+ * SSE math works with doubles and floats natively, so is free from this
+ * instability, while a bit less precise.
+ * We aren't requiring C99 yet, so use GCC's define instead of the C99-standard
+ * "FLT_EVAL_METHOD" from float.h for deciding which mode is used.
+ */
+
+#undef NATIVE_DOUBLES
+#if defined(__FLT_EVAL_METHOD__) && ((__FLT_EVAL_METHOD__ == 0) || (__FLT_EVAL_METHOD__ == 1))
+#define NATIVE_DOUBLES
+#endif
+
+/*
+ * rint() function rounds halfway cases (0.5 etc.) to even, which may cause
+ * weird results in geometry calculations. And straightforward (int)(X + 0.5)
+ * may be affected by double-rounding issues on x87 FPU - and in case floor()
+ * is implemented as compiler intrinsic, the same can happen with it too.
+ * These macros are for when neither is acceptable.
+ */
+#ifndef NATIVE_DOUBLES /* Have extra precision */
+#define WJ_ROUND(I,X) \
+{						\
+	const volatile double RounD___ = (X);	\
+	(I) = (int)(RounD___ + 0.5);		\
+}
+#define WJ_FLOOR(I,X) \
+{						\
+	const volatile double RounD___ = (X);	\
+	(I) = floor(RounD___);			\
+}
+#else /* Doubles are doubles */
+#define WJ_ROUND(I,X) (I) = (int)((X) + 0.5)
+#define WJ_FLOOR(I,X) (I) = floor(X)
+#endif
