@@ -1,5 +1,5 @@
 /*	mygtk.h
-	Copyright (C) 2004-2008 Mark Tyler and Dmitry Groshev
+	Copyright (C) 2004-2009 Mark Tyler and Dmitry Groshev
 
 	This file is part of mtPaint.
 
@@ -40,6 +40,8 @@ GtkWidget *add_a_toggle( char *label, GtkWidget *box, gboolean value );
 GtkWidget *add_to_table( char *text, GtkWidget *table, int row, int column, int spacing);
 GtkWidget *spin_to_table( GtkWidget *table, int row, int column, int spacing,
 	int value, int min, int max );
+GtkWidget *float_spin_to_table(GtkWidget *table, int row, int column, int spacing,
+	double value, double min, double max);
 void add_hseparator( GtkWidget *widget, int xs, int ys );
 
 void progress_init(char *text, int canc);		// Initialise progress window
@@ -83,6 +85,12 @@ int read_spin(GtkWidget *spin);
 double read_float_spin(GtkWidget *spin);
 GtkWidget *add_float_spin(double value, double min, double max);
 void spin_connect(GtkWidget *spin, GtkSignalFunc handler, gpointer user_data);
+#if GTK_MAJOR_VERSION == 1
+void spin_set_range(GtkWidget *spin, int min, int max);
+#else
+#define spin_set_range(spin, min, max) \
+	gtk_spin_button_set_range(GTK_SPIN_BUTTON(spin), (min), (max))
+#endif
 
 // Box unpacking macros
 #define BOX_CHILD_0(box) \
@@ -158,12 +166,17 @@ GtkWidget *sig_toggle_button(char *label, int value, gpointer var, GtkSignalFunc
 
 void clist_enable_drag(GtkWidget *clist);
 
+// Move browse-mode selection in GtkCList without invoking callbacks
+
+void clist_reselect_row(GtkCList *clist, int n);
+
 // Properly destroy transient window
 
 void destroy_dialog(GtkWidget *window);
 
 // Settings notebook
 
+GtkWidget *plain_book(GtkWidget **pages, int npages);
 GtkWidget *buttoned_book(GtkWidget **page0, GtkWidget **page1,
 	GtkWidget **button, char *button_label);
 
@@ -174,6 +187,7 @@ GtkWidget *xpack(GtkWidget *box, GtkWidget *widget);
 GtkWidget *pack_end(GtkWidget *box, GtkWidget *widget);
 GtkWidget *pack5(GtkWidget *box, GtkWidget *widget);
 GtkWidget *xpack5(GtkWidget *box, GtkWidget *widget);
+GtkWidget *pack_end5(GtkWidget *box, GtkWidget *widget);
 
 // Save/restore window positions
 
@@ -185,10 +199,6 @@ void win_restore_pos(GtkWidget *window, char *inikey, int defx, int defy,
 
 void viewport_style(GtkWidget *widget);
 
-// Eliminate flicker when scrolling
-
-void fix_scroll(GtkWidget *scroll);
-
 // Fix for paned widgets losing focus in GTK+1
 
 #if GTK_MAJOR_VERSION == 1
@@ -197,6 +207,11 @@ void paned_mouse_fix(GtkWidget *widget);
 #define paned_mouse_fix(X)
 #endif
 
+// Eliminate flicker when scrolling
+
+void fix_vport(GtkWidget *vport);
+#define fix_scroll(scroll) fix_vport(GTK_BIN(scroll)->child)
+
 // Init-time bugfixes
 
 void gtk_init_bugfixes();
@@ -215,6 +230,11 @@ guint keyval_key(guint keyval);
 
 int arrow_key(GdkEventKey *event, int *dx, int *dy, int mult);
 
+// Create pixmap cursor
+
+GdkCursor *make_cursor(const char *icon, const char *mask, int w, int h,
+	int tip_x, int tip_y);
+
 // Focusable pixmap widget
 
 GtkWidget *wj_fpixmap(int width, int height);
@@ -242,6 +262,44 @@ GtkWidget *wj_size_bin();
 
 gpointer toggle_updates(GtkWidget *widget, gpointer unlock);
 
+// Drawable to RGB
+
+unsigned char *wj_get_rgb_image(GdkWindow *window, GdkPixmap *pixmap,
+	unsigned char *buf, int x, int y, int width, int height);
+
+// Clipboard
+
+int internal_clipboard(int which);
+int process_clipboard(int which, char *what, GtkSignalFunc handler, gpointer data);
+int offer_clipboard(int which, GtkTargetEntry *targets, int ntargets,
+	GtkSignalFunc handler);
+
+// Allocate a memory chunk which is freed along with a given widget
+
+void *bound_malloc(GtkWidget *widget, int size);
+
+// Gamma correction toggle
+
+GtkWidget *gamma_toggle();
+
+// Image widget
+
+GtkWidget *xpm_image(char **xpm);
+
+// Render stock icons to pixmaps
+
+/* !!! Mask needs be zeroed before the call - especially with GTK+1 :-) */
+#if GTK_MAJOR_VERSION == 1
+#define render_stock_pixmap(X,Y,Z) NULL
+#else
+GdkPixmap *render_stock_pixmap(GtkWidget *widget, const gchar *stock_id,
+	GdkBitmap **mask);
+#endif
+
+// Release outstanding pointer grabs
+
+int release_grab();
+
 // Filtering bogus xine-ui "keypresses" (Linux only)
 #ifdef WIN32
 #define XINE_FAKERY(key) 0
