@@ -1,5 +1,5 @@
 /*	prefs.c
-	Copyright (C) 2005-2008 Mark Tyler and Dmitry Groshev
+	Copyright (C) 2005-2009 Mark Tyler and Dmitry Groshev
 
 	This file is part of mtPaint.
 
@@ -32,8 +32,8 @@
 ///	PREFERENCES WINDOW
 
 GtkWidget *prefs_window, *prefs_status[STATUS_ITEMS];
-static GtkWidget *spinbutton_maxmem, *spinbutton_maxundo, *spinbutton_greys,
-	*spinbutton_nudge, *spinbutton_pan;
+static GtkWidget *spinbutton_maxmem, *spinbutton_maxundo, *spinbutton_commundo;
+static GtkWidget *spinbutton_greys, *spinbutton_nudge, *spinbutton_pan;
 static GtkWidget *spinbutton_trans, *spinbutton_hotx, *spinbutton_hoty,
 	*spinbutton_jpeg, *spinbutton_jp2, *spinbutton_png, *spinbutton_recent,
 	*spinbutton_silence;
@@ -43,8 +43,10 @@ static GtkWidget *checkbutton_tgaRLE, *checkbutton_tga565, *checkbutton_tgadef,
 static GtkWidget *checkbutton_paste, *checkbutton_cursor, *checkbutton_exit, *checkbutton_quit;
 static GtkWidget *checkbutton_zoom[4],		// zoom 100%, wheel, optimize cheq, disable trans
 	*checkbutton_commit, *checkbutton_center, *checkbutton_gamma;
+#if GTK_MAJOR_VERSION == 2
+static GtkWidget *checkbutton_menuicons;
+#endif
 static GtkWidget *clipboard_entry, *entry_handbook[2], *entry_def[2];
-static GtkWidget *spinbutton_grid[4];
 static GtkWidget *check_tablet[3], *hscale_tablet[3], *label_tablet_device, *label_tablet_pressure;
 
 static char	*tablet_ini[] = { "tablet_value_size", "tablet_value_flow", "tablet_value_opacity" },
@@ -65,36 +67,27 @@ float tablet_tool_factor[3];			// Size, flow, opacity
 
 #ifdef U_NLS
 
-#define PREF_LANGS 16
+#define PREF_LANGS 19
 
 char *pref_lang_ini_code[PREF_LANGS] = { "system",
 	"zh_CN.utf8", "zh_TW.utf8",
-	"cs_CZ", "en_GB", "fr_FR", "gl_ES",
-	"de_DE", "ja_JP.utf8", "pl_PL", "pt_PT",
-	"pt_BR", "ru_RU", "sk_SK",
-	"es_ES", "tr_TR" };
+	"cs_CZ", "nl_NL", "en_GB", "fr_FR",
+	"gl_ES", "de_DE", "it_IT", "ja_JP.utf8",
+	"pl_PL", "pt_PT", "pt_BR",
+	"ru_RU", "sk_SK", "es_ES", "sv_SE",
+	"tr_TR" };
 
 int pref_lang;
 
 #endif
 
 
-static gint expose_tablet_preview( GtkWidget *widget, GdkEventExpose *event )
+static gboolean expose_tablet_preview(GtkWidget *widget, GdkEventExpose *event)
 {
-	unsigned char *rgb;
-	int i, x = event->area.x, y = event->area.y, w = event->area.width, h = event->area.height;
+        gdk_draw_rectangle(widget->window, widget->style->white_gc, TRUE,
+		event->area.x, event->area.y, event->area.width, event->area.height);
 
-	rgb = malloc( w*h*3 );
-	if ( rgb == NULL ) return FALSE;
-
-	for ( i=0; i<(w*h*3); i++ ) rgb[i] = 255;		// Pure white
-
-	gdk_draw_rgb_image (widget->window, widget->style->black_gc,
-			x, y, w, h, GDK_RGB_DITHER_NONE, rgb, w*3 );
-
-	free( rgb );
-
-	return FALSE;
+	return (FALSE);
 }
 
 
@@ -264,6 +257,7 @@ static void prefs_apply(GtkWidget *widget)
 
 	mem_undo_limit = read_spin(spinbutton_maxmem);
 	mem_undo_depth = read_spin(spinbutton_maxundo);
+	mem_undo_common = read_spin(spinbutton_commundo);
 	mem_background = read_spin(spinbutton_greys);
 	mem_nudge = read_spin(spinbutton_nudge);
 	max_pan = read_spin(spinbutton_pan);
@@ -277,10 +271,6 @@ static void prefs_apply(GtkWidget *widget)
 	recent_files = read_spin(spinbutton_recent);
 	silence_limit = read_spin(spinbutton_silence);
 
-	mem_grid_min = read_spin(spinbutton_grid[0]);
-	for ( i=0; i<3; i++ )
-		mem_grid_rgb[i] = read_spin(spinbutton_grid[i + 1]);
-
 	for (i = 0; i < 3; i++)
 	{
 		tablet_tool_use[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_tablet[i]));
@@ -290,10 +280,6 @@ static void prefs_apply(GtkWidget *widget)
 		tablet_tool_factor[i] = j / 100.0;
 	}
 
-	inifile_set_gint32( "gridR", mem_grid_rgb[0] );
-	inifile_set_gint32( "gridG", mem_grid_rgb[1] );
-	inifile_set_gint32( "gridB", mem_grid_rgb[2] );
-
 
 	tga_RLE = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_tgaRLE)) ? 1 : 0;
 	tga_565 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_tga565));
@@ -314,8 +300,7 @@ static void prefs_apply(GtkWidget *widget)
 	chequers_optimize = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_zoom[2]));
 	opaque_view = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_zoom[3]));
 
-	inifile_set_gboolean( "pasteCommit",
-		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_commit)) );
+	paste_commit = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_commit));
 
 	q_quit = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_quit));
 
@@ -323,6 +308,10 @@ static void prefs_apply(GtkWidget *widget)
 		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_center)));
 	inifile_set_gboolean("defaultGamma",
 		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_gamma)));
+#if GTK_MAJOR_VERSION == 2
+	show_menu_icons = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_menuicons));
+#endif
+
 
 #ifdef U_NLS
 	inifile_set("languageSETTING", pref_lang_ini_code[pref_lang]);
@@ -349,12 +338,7 @@ static void prefs_apply(GtkWidget *widget)
 		load_def_patterns(path);
 	inifile_set(DEFAULT_PAT_INI, path);
 
-	update_undo_depth();		// If undo depth was changed
-	update_all_views();		// Update canvas for changes
-	set_cursor();
-
-	update_recent_files();
-	init_status_bar();
+	update_stuff(UPD_PREFS);
 }
 
 static void prefs_ok(GtkWidget *widget)
@@ -411,30 +395,27 @@ static gint tablet_preview_motion(GtkWidget *widget, GdkEventMotion *event)
 	return TRUE;
 }
 
-void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
+void pressed_preferences()
 {
 	int i;
 #ifdef U_NLS
 	char *pref_langs[PREF_LANGS] = { _("Default System Language"),
 		_("Chinese (Simplified)"), _("Chinese (Taiwanese)"),
-		_("Czech"), _("English (UK)"), _("French"), _("Galician"),
-		_("German"), _("Japanese"), _("Polish"), _("Portuguese"),
-		_("Portuguese (Brazilian)"), _("Russian"), _("Slovak"),
-		_("Spanish"), _("Turkish") };
+		_("Czech"), _("Dutch"), _("English (UK)"), _("French"),
+		_("Galician"), _("German"), _("Italian"), _("Japanese"),
+		_("Polish"), _("Portuguese"), _("Portuguese (Brazilian)"),
+		_("Russian"), _("Slovak"), _("Spanish"), _("Swedish"),
+		_("Turkish") };
 #endif
 
 
-	GtkWidget *vbox3, *hbox4, *table3, *table4, *table5, *drawingarea_tablet;
+	GtkWidget *vbox3, *hbox4, *table3, *table4, *drawingarea_tablet;
 	GtkWidget *button1, *notebook1, *page, *vbox_2, *label;
 
-	char *tab_tex[] = { _("Max memory used for undo (MB)"),
-		_("Max undo levels"), _("Greyscale backdrop"),
-		_("Selection nudge pixels"), _("Max Pan Window Size") };
 	char *tab_tex2[] = { _("Transparency index"), _("XBM X hotspot"), _("XBM Y hotspot"),
 		_("JPEG Save Quality (100=High)"), _("JPEG2000 Compression (0=Lossless)"),
 		_("PNG Compression (0=None)"), _("Recently Used Files"),
 		_("Progress bar silence limit") };
-	char *tab_tex3[] = { _("Minimum grid zoom"), _("Grid colour RGB") };
 	char *stat_tex[] = { _("Canvas Geometry"), _("Cursor X,Y"),
 		_("Pixel [I] {RGB}"), _("Selection Geometry"), _("Undo / Redo") },
 		*tablet_txt[] = { _("Size"), _("Flow"), _("Opacity") };
@@ -453,24 +434,66 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 ///	SETUP NOTEBOOK
 
 	notebook1 = xpack(vbox3, gtk_notebook_new());
-	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook1), GTK_POS_TOP);
-	gtk_widget_show (notebook1);
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook1), GTK_POS_TOP);
+//	gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook1), TRUE);
+	gtk_widget_show(notebook1);
 
 ///	---- TAB1 - GENERAL
 
 	page = add_new_page(notebook1, _("General"));
-	table3 = add_a_table( 5, 2, 10, page );
+	table3 = add_a_table(3, 2, 10, page);
 
 ///	TABLE TEXT
-	for (i = 0; i < 5; i++ ) add_to_table( tab_tex[i], table3, i, 0, 5 );
+	add_to_table(_("Max memory used for undo (MB)"), table3, 0, 0, 5);
+	add_to_table(_("Max undo levels"), table3, 1, 0, 5);
+	add_to_table(_("Communal layer undo space (%)"), table3, 2, 0, 5);
 
 ///	TABLE SPINBUTTONS
 	spinbutton_maxmem = spin_to_table(table3, 0, 1, 5, mem_undo_limit, 1, 2048);
 	spinbutton_maxundo = spin_to_table(table3, 1, 1, 5, mem_undo_depth & ~1,
 		MIN_UNDO & ~1, MAX_UNDO & ~1);
-	spinbutton_greys = spin_to_table(table3, 2, 1, 5, mem_background, 0, 255);
-	spinbutton_nudge = spin_to_table(table3, 3, 1, 5, mem_nudge, 2, 512);
-	spinbutton_pan = spin_to_table(table3, 4, 1, 5, max_pan, 64, 256);
+	spinbutton_commundo = spin_to_table(table3, 2, 1, 5, mem_undo_common, 0, 100);
+
+	checkbutton_gamma = add_a_toggle(_("Use gamma correction by default"),
+		page, inifile_get_gboolean("defaultGamma", FALSE));
+	checkbutton_zoom[2] = add_a_toggle( _("Optimize alpha chequers"),
+		page, chequers_optimize );
+	checkbutton_zoom[3] = add_a_toggle( _("Disable view window transparencies"),
+		page, opaque_view );
+
+///	LANGUAGE SWITCHBOX
+#ifdef U_NLS
+	vbox_2 = gtk_vbox_new(FALSE, 5);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox_2), 5);
+
+	label = pack(vbox_2, gtk_label_new( _("Select preferred language translation\n\n"
+		"You will need to restart mtPaint\nfor this to take full effect")));
+
+	for (i = 0; i < PREF_LANGS; i++)
+	{
+		if (!strcmp(pref_lang_ini_code[i],
+			inifile_get("languageSETTING", "system"))) break;
+	}
+	pack(vbox_2, wj_option_menu(pref_langs, PREF_LANGS, i, &pref_lang, NULL));
+
+	gtk_widget_show_all(vbox_2);
+	add_with_frame(page, _("Language"), vbox_2, 5);
+#endif
+
+///	---- TAB2 - INTERFACE
+
+	page = add_new_page(notebook1, _("Interface"));
+	table3 = add_a_table(3, 2, 10, page);
+
+///	TABLE TEXT
+	add_to_table(_("Greyscale backdrop"), table3, 0, 0, 5);
+	add_to_table(_("Selection nudge pixels"), table3, 1, 0, 5);
+	add_to_table(_("Max Pan Window Size"), table3, 2, 0, 5);
+
+///	TABLE SPINBUTTONS
+	spinbutton_greys = spin_to_table(table3, 0, 1, 5, mem_background, 0, 255);
+	spinbutton_nudge = spin_to_table(table3, 1, 1, 5, mem_nudge, 2, MAX_WIDTH);
+	spinbutton_pan = spin_to_table(table3, 2, 1, 5, max_pan, 64, 256);
 
 	checkbutton_paste = add_a_toggle( _("Display clipboard while pasting"),
 		page, show_paste );
@@ -480,15 +503,19 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 		page, inifile_get_gboolean("exitToggle", FALSE) );
 	checkbutton_quit = add_a_toggle( _("Q key quits mtPaint"),
 		page, q_quit );
-	checkbutton_commit = add_a_toggle( _("Changing tool commits paste"),
-		page, inifile_get_gboolean("pasteCommit", FALSE) );
+	checkbutton_commit = add_a_toggle(_("Changing tool commits paste"),
+		page, paste_commit);
 	checkbutton_center = add_a_toggle(_("Centre tool settings dialogs"),
 		page, inifile_get_gboolean("centerSettings", TRUE));
-	checkbutton_gamma = add_a_toggle(_("Use gamma correction by default"),
-		page, inifile_get_gboolean("defaultGamma", FALSE));
+	checkbutton_zoom[0] = add_a_toggle( _("New image sets zoom to 100%"),
+		page, inifile_get_gboolean("zoomToggle", FALSE) );
+#if GTK_MAJOR_VERSION == 2
+	checkbutton_zoom[1] = add_a_toggle( _("Mouse Scroll Wheel = Zoom"),
+		page, inifile_get_gboolean("scrollwheelZOOM", FALSE) );
+	checkbutton_menuicons = add_a_toggle(_("Use menu icons"), page, show_menu_icons);
+#endif
 
-
-///	---- TAB2 - FILES
+///	---- TAB3 - FILES
 
 	page = add_new_page(notebook1, _("Files"));
 	table4 = add_a_table(7, 2, 10, page);
@@ -500,7 +527,7 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 	spinbutton_hotx    = spin_to_table(table4, 1, 1, 4, mem_xbm_hot_x, -1, mem_width - 1);
 	spinbutton_hoty    = spin_to_table(table4, 2, 1, 4, mem_xbm_hot_y, -1, mem_height - 1);
 	spinbutton_jpeg    = spin_to_table(table4, 3, 1, 4, jpeg_quality, 0, 100);
-	spinbutton_jp2     = spin_to_table(table4, 4, 1, 4, jp2_rate, 1, 100);
+	spinbutton_jp2     = spin_to_table(table4, 4, 1, 4, jp2_rate, 0, 100);
 	spinbutton_png     = spin_to_table(table4, 5, 1, 4, png_compression, 0, 9);
 	spinbutton_recent  = spin_to_table(table4, 6, 1, 4, recent_files, 0, MAX_RECENT);
 	spinbutton_silence = spin_to_table(table4, 7, 1, 4, silence_limit, 0, 28);
@@ -509,7 +536,7 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 	checkbutton_tgadef = add_a_toggle(_("Write TGAs in bottom-up row order"), page, tga_defdir);
 	checkbutton_undo   = add_a_toggle(_("Undoable image loading"), page, undo_load);
 
-///	---- TAB3 - PATHS
+///	---- TAB4 - PATHS
 
 	page = add_new_page(notebook1, _("Paths"));
 
@@ -539,7 +566,7 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 	gtk_entry_set_text(GTK_ENTRY(entry_def[1]), txt);
 
 
-///	---- TAB4 - STATUS BAR
+///	---- TAB5 - STATUS BAR
 
 	page = add_new_page(notebook1, _("Status Bar"));
 
@@ -548,35 +575,6 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 		prefs_status[i] = add_a_toggle( stat_tex[i], page, status_on[i] );
 	}
 
-///	---- TAB5 - ZOOM
-
-	page = add_new_page(notebook1, _("Zoom"));
-	table5 = add_a_table( 2, 4, 10, page );
-
-///	TABLE TEXT
-	for ( i=0; i<2; i++ ) add_to_table( tab_tex3[i], table5, i, 0, 5 );
-
-///	TABLE SPINBUTTONS
-	spinbutton_grid[0] = spin_to_table(table5, 0, 1, 5, mem_grid_min, 2, 12);
-	for (i = 0; i < 3; i++)
-	{
-		spinbutton_grid[i + 1] = spin_to_table(table5, 1, i + 1, 5,
-			mem_grid_rgb[i], 0, 255);
-	}
-
-	checkbutton_zoom[0] = add_a_toggle( _("New image sets zoom to 100%"),
-		page, inifile_get_gboolean("zoomToggle", FALSE) );
-#if GTK_MAJOR_VERSION == 2
-	checkbutton_zoom[1] = add_a_toggle( _("Mouse Scroll Wheel = Zoom"),
-		page, inifile_get_gboolean("scrollwheelZOOM", FALSE) );
-#endif
-	checkbutton_zoom[2] = add_a_toggle( _("Optimize alpha chequers"),
-		page, chequers_optimize );
-	checkbutton_zoom[3] = add_a_toggle( _("Disable view window transparencies"),
-		page, opaque_view );
-
-
-
 ///	---- TAB6 - TABLET
 
 	page = add_new_page(notebook1, _("Tablet"));
@@ -599,7 +597,7 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 
 	label = add_to_table( _("Tool Variable"), table3, 0, 0, 0 );
 	gtk_misc_set_padding (GTK_MISC (label), 5, 5);
-	snprintf(txt, 60, "%s, %%", _("Factor"));
+	snprintf(txt, 60, "%s (%%)", _("Factor"));
 	label = add_to_table( txt, table3, 0, 1, 0 );
 	gtk_misc_set_padding (GTK_MISC (label), 5, 5);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.4, 0.5);
@@ -657,31 +655,6 @@ void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data )
 
 
 
-///	---- TAB7 - LANGUAGE
-
-#ifdef U_NLS
-
-	page = add_new_page(notebook1, _("Language"));
-	gtk_container_set_border_width(GTK_CONTAINER(page), 10);
-
-	add_hseparator( page, 200, 10 );
-	label = pack(page, gtk_label_new( _("Select preferred language translation\n\n"
-				"You will need to restart mtPaint\nfor this to take full effect")));
-	gtk_widget_show (label);
-	add_hseparator( page, 200, 10 );
-
-	for (i = 0; i < PREF_LANGS; i++)
-	{
-		if (!strcmp(pref_lang_ini_code[i],
-			inifile_get("languageSETTING", "system"))) break;
-	}
-	xpack(page, wj_radio_pack(pref_langs, PREF_LANGS, (PREF_LANGS + 1) / 2,
-		i, &pref_lang, NULL));
-
-#endif
-
-
-
 ///	Bottom of Prefs window
 
 	hbox4 = pack(vbox3, OK_box(0, prefs_window, _("OK"), GTK_SIGNAL_FUNC(prefs_ok),
