@@ -1,11 +1,11 @@
 /*	prefs.h
-	Copyright (C) 2005 Mark Tyler
+	Copyright (C) 2005-2008 Mark Tyler
 
 	This file is part of mtPaint.
 
 	mtPaint is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2 of the License, or
+	the Free Software Foundation; either version 3 of the License, or
 	(at your option) any later version.
 
 	mtPaint is distributed in the hope that it will be useful,
@@ -17,16 +17,13 @@
 	along with mtPaint in the file COPYING.
 */
 
-#include <gtk/gtk.h>
-
-
 #define HANDBOOK_BROWSER_INI "docsBrowser"
 #define HANDBOOK_LOCATION_INI "docsLocation"
+#define DEFAULT_PAL_INI "defaultPalette"
+#define DEFAULT_PAT_INI "defaultPatterns"
 
-GtkWidget *clipboard_entry, *entry_handbook[2];
-
-gboolean tablet_working, tablet_tool_use[3];	// Size, flow, opacity
+int tablet_working, tablet_tool_use[3];		// Size, flow, opacity
 float tablet_tool_factor[3];			// Size, flow, opacity
 
-void pressed_preferences( GtkMenuItem *menu_item, gpointer user_data );
+void pressed_preferences();
 void init_tablet();				// Set up variables
