@@ -1,5 +1,5 @@
 /*	spawn.h
-	Copyright (C) 2007 Mark Tyler
+	Copyright (C) 2007-2008 Mark Tyler and Dmitry Groshev
 
 	This file is part of mtPaint.
 
@@ -29,7 +29,16 @@ int spawn_expansion(char *cline, char *directory);
 		// Replace %f with "current filename", then run via shell
 
 void pressed_file_configure();
-void pressed_file_action( GtkMenuItem *menu_item, gpointer user_data, gint item );
+void pressed_file_action(int item);
 void init_factions();					// Initialize file action menu
 
 void spawn_quit();	// Delete temp files
+
+// Default action codes
+#define DA_GIF_CREATE  0
+#define DA_GIF_EXPLODE 1
+#define DA_GIF_PLAY    2
+#define DA_GIF_EDIT    3
+
+//	Run some default action
+int run_def_action(int action, char *sname, char *dname, int delay);
