@@ -1,5 +1,5 @@
 /*	toolbar.c
-	Copyright (C) 2006-2008 Mark Tyler and Dmitry Groshev
+	Copyright (C) 2006-2009 Mark Tyler and Dmitry Groshev
 
 	This file is part of mtPaint.
 
@@ -32,102 +32,9 @@
 #include "channels.h"
 #include "csel.h"
 #include "font.h"
+#include "icons.h"
 
 
-#include "graphics/xpm_paint.xpm"
-
-#include "graphics/xbm_square.xbm"
-#include "graphics/xbm_square_mask.xbm"
-
-#include "graphics/xbm_circle.xbm"
-#include "graphics/xbm_circle_mask.xbm"
-
-#include "graphics/xbm_horizontal.xbm"
-#include "graphics/xbm_horizontal_mask.xbm"
-
-#include "graphics/xbm_vertical.xbm"
-#include "graphics/xbm_vertical_mask.xbm"
-
-#include "graphics/xbm_slash.xbm"
-#include "graphics/xbm_slash_mask.xbm"
-
-#include "graphics/xbm_backslash.xbm"
-#include "graphics/xbm_backslash_mask.xbm"
-
-#include "graphics/xbm_spray.xbm"
-#include "graphics/xbm_spray_mask.xbm"
-
-#include "graphics/xpm_shuffle.xpm"
-#include "graphics/xbm_shuffle.xbm"
-#include "graphics/xbm_shuffle_mask.xbm"
-
-#include "graphics/xpm_flood.xpm"
-#include "graphics/xbm_flood.xbm"
-#include "graphics/xbm_flood_mask.xbm"
-
-#include "graphics/xpm_line.xpm"
-#include "graphics/xbm_line.xbm"
-#include "graphics/xbm_line_mask.xbm"
-
-#include "graphics/xpm_select.xpm"
-#include "graphics/xbm_select.xbm"
-#include "graphics/xbm_select_mask.xbm"
-
-#include "graphics/xpm_smudge.xpm"
-#include "graphics/xbm_smudge.xbm"
-#include "graphics/xbm_smudge_mask.xbm"
-
-#include "graphics/xpm_polygon.xpm"
-#include "graphics/xbm_polygon.xbm"
-#include "graphics/xbm_polygon_mask.xbm"
-
-#include "graphics/xpm_clone.xpm"
-#include "graphics/xbm_clone.xbm"
-#include "graphics/xbm_clone_mask.xbm"
-
-#include "graphics/xbm_move.xbm"
-#include "graphics/xbm_move_mask.xbm"
-
-#include "graphics/xpm_brcosa.xpm"
-#include "graphics/xpm_flip_vs.xpm"
-#include "graphics/xpm_flip_hs.xpm"
-#include "graphics/xpm_rotate_cs.xpm"
-#include "graphics/xpm_rotate_as.xpm"
-#include "graphics/xpm_text.xpm"
-#include "graphics/xpm_lasso.xpm"
-
-#include "graphics/xpm_ellipse.xpm"
-#include "graphics/xpm_ellipse2.xpm"
-#include "graphics/xpm_rect1.xpm"
-#include "graphics/xpm_rect2.xpm"
-#include "graphics/xpm_pan.xpm"
-
-#include "graphics/xpm_new.xpm"
-#include "graphics/xpm_open.xpm"
-#include "graphics/xpm_save.xpm"
-#include "graphics/xpm_cut.xpm"
-#include "graphics/xpm_copy.xpm"
-#include "graphics/xpm_paste.xpm"
-#include "graphics/xpm_undo.xpm"
-#include "graphics/xpm_redo.xpm"
-
-#include "graphics/xpm_up.xpm"
-#include "graphics/xpm_down.xpm"
-#include "graphics/xpm_centre.xpm"
-#include "graphics/xpm_close.xpm"
-
-#include "graphics/xpm_mode_cont.xpm"
-#include "graphics/xpm_mode_opac.xpm"
-#include "graphics/xpm_mode_tint.xpm"
-#include "graphics/xpm_mode_tint2.xpm"
-#include "graphics/xpm_mode_csel.xpm"
-#include "graphics/xpm_mode_blend.xpm"
-#include "graphics/xpm_mode_mask.xpm"
-
-#include "graphics/xpm_grad_place.xpm"
-#include "graphics/xbm_grad.xbm"
-#include "graphics/xbm_grad_mask.xbm"
-
 
 GtkWidget *icon_buttons[TOTAL_ICONS_TOOLS];
 
@@ -142,64 +49,12 @@ GdkCursor *m_cursor[32];		// My mouse cursors
 
 static GtkWidget *toolbar_zoom_main, *toolbar_zoom_view,
 	*toolbar_labels[2],		// Colour A & B details
-	*ts_spinslides[3],		// Size, flow, opacity
-	*tb_label_opacity		// Opacity label
-	;
+	*ts_spinslides[4],		// Size, flow, opacity, value
+	*ts_label_channel;		// Channel name
+
 static unsigned char mem_prev[PREVIEW_WIDTH * PREVIEW_HEIGHT * 3];
 					// RGB colours, tool, pattern preview
 
-typedef struct
-{
-	unsigned char ID;
-	signed char radio;
-	unsigned char sep, rclick;
-	int actmap;
-	char *tooltip, **xpm;
-	GtkWidget *widget;
-} toolbar_item;
-
-static void fill_toolbar(GtkToolbar *bar, toolbar_item *items,
-	GtkSignalFunc lclick, int lbase, GtkSignalFunc rclick, int rbase);
-
-#undef _
-#define _(X) X
-
-static toolbar_item layer_bar[] = {
-	{ LTB_NEW,    -1, 0, 0, 0, _("New Layer"), xpm_new_xpm },
-	{ LTB_RAISE,  -1, 0, 0, 0, _("Raise"), xpm_up_xpm },
-	{ LTB_LOWER,  -1, 0, 0, 0, _("Lower"), xpm_down_xpm },
-	{ LTB_DUP,    -1, 0, 0, 0, _("Duplicate Layer"), xpm_copy_xpm },
-	{ LTB_CENTER, -1, 0, 0, 0, _("Centralise Layer"), xpm_centre_xpm },
-	{ LTB_DEL,    -1, 0, 0, 0, _("Delete Layer"), xpm_cut_xpm },
-	{ LTB_CLOSE,  -1, 0, 0, 0, _("Close Layers Window"), xpm_close_xpm },
-	{ 0, 0, 0, 0, 0, NULL, NULL }};
-
-#undef _
-#define _(X) __(X)
-
-/* Create toolbar for layers window */
-GtkWidget *layer_toolbar(GtkWidget **wlist)
-{		
-	int i;
-	GtkWidget *toolbar;
-
-#if GTK_MAJOR_VERSION == 1
-	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
-#endif
-#if GTK_MAJOR_VERSION == 2
-	toolbar = gtk_toolbar_new();
-	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
-#endif
-	fill_toolbar(GTK_TOOLBAR(toolbar), layer_bar,
-		GTK_SIGNAL_FUNC(layer_iconbar_click), 0, NULL, 0);
-	gtk_widget_show(toolbar);
-
-	for (i = 0; i < TOTAL_ICONS_LAYER; i++)
-		wlist[i] = layer_bar[i].widget;
-
-	return toolbar;
-}
-
 static gint expose_preview( GtkWidget *widget, GdkEventExpose *event )
 {
 	int rx, ry, rw, rh;
@@ -234,7 +89,7 @@ static gint click_colours( GtkWidget *widget, GdkEventButton *event )
 		if ( event->y > 31 ) choose_pattern(0);
 		else
 		{
-			if ( event->x < 48 ) choose_colours();
+			if ( event->x < 48 ) colour_selector(COLSEL_EDIT_AB);
 			else choose_pattern(1);
 		}
 	}
@@ -242,7 +97,7 @@ static gint click_colours( GtkWidget *widget, GdkEventButton *event )
 	return FALSE;
 }
 
-static GtkWidget *toolbar_add_zoom(GtkWidget *box)		// Add zoom combo box
+static GtkWidget *toolbar_add_zoom(GtkWidget *tbar)	// Add zoom combo box
 {
 	int i;
 	static char *txt[] = { "10%", "20%", "25%", "33%", "50%", "100%",
@@ -251,7 +106,7 @@ static GtkWidget *toolbar_add_zoom(GtkWidget *box)		// Add zoom combo box
 	GList *combo_list = NULL;
 
 
-	combo = pack(box, gtk_combo_new());
+	combo = gtk_combo_new();
 	gtk_combo_set_value_in_list (GTK_COMBO (combo), FALSE, FALSE);
 	gtk_widget_show (combo);
 	combo_entry = GTK_COMBO (combo)->entry;
@@ -274,6 +129,9 @@ static GtkWidget *toolbar_add_zoom(GtkWidget *box)		// Add zoom combo box
 	g_list_free( combo_list );
 	gtk_entry_set_text( GTK_ENTRY(combo_entry), "100%" );
 
+	gtk_toolbar_append_element(GTK_TOOLBAR(tbar), GTK_TOOLBAR_CHILD_WIDGET,
+		combo, NULL, NULL, NULL, NULL, NULL, NULL);
+
 	return combo;
 }
 
@@ -295,10 +153,7 @@ static float toolbar_get_zoom( GtkWidget *combo )
 
 void toolbar_viewzoom(gboolean visible)
 {
-	if ( visible )
-		gtk_widget_show( toolbar_zoom_view );
-	else
-		gtk_widget_hide( toolbar_zoom_view );
+	(visible ? gtk_widget_show : gtk_widget_hide)(toolbar_zoom_view);
 }
 
 void toolbar_zoom_update()			// Update the zoom combos to reflect current zoom
@@ -329,6 +184,7 @@ static void toolbar_zoom_view_change()
 	if ( new > 0 ) vw_align_size( new );
 }
 
+static GtkWidget *settings_buttons[TOTAL_SETTINGS];
 static int *vars_settings[TOTAL_SETTINGS] = {
 	&mem_continuous, &mem_undo_opacity,
 	&tint_mode[0], &tint_mode[1],
@@ -336,28 +192,19 @@ static int *vars_settings[TOTAL_SETTINGS] = {
 	&mem_unmask, &mem_gradient
 };
 
-void toolbar_mode_change(GtkWidget *widget, gpointer data)
+void mode_change(int setting, int state)
 {
-	gint j = (gint) data;
+	if (!GTK_TOGGLE_BUTTON(settings_buttons[setting])->active != !state) // Toggle the button
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(settings_buttons[setting]), state);
 
-	*(vars_settings[j]) = !!GTK_TOGGLE_BUTTON(widget)->active;
-
-	switch (j)
+	if (!state == !*vars_settings[setting]) return; // No change, or changed already
+	*(vars_settings[setting]) = state;
+	if ((setting == SETB_CSEL) && mem_cselect && !csel_data)
 	{
-	case SETB_CSEL:
-		if (mem_cselect && !csel_data)
-		{
-			csel_init();
-			mem_cselect = !!csel_data;
-		}
-		break;
-	case SETB_GRAD:
-		if ((gradient[mem_channel].status == GRAD_DONE) &&
-			(tool_type != TOOL_GRADIENT))
-			repaint_grad(mem_gradient);
-		break;
+		csel_init();
+		mem_cselect = !!csel_data;
 	}
-	gtk_widget_queue_draw(drawing_canvas);
+	update_stuff(setting == SETB_GRAD ? UPD_GMODE : UPD_MODE);
 }
 
 static int set_flood(GtkWidget *box, gpointer fdata)
@@ -380,6 +227,17 @@ static int set_flood(GtkWidget *box, gpointer fdata)
 	return TRUE;
 }
 
+void flood_settings() /* Flood fill step */
+{
+	GtkWidget *box = gtk_vbox_new(FALSE, 5);
+	gtk_widget_show(box);
+	pack(box, add_float_spin(flood_step, 0, 200));
+	add_a_toggle(_("RGB Cube"), box, flood_cube);
+	add_a_toggle(_("By image channel"), box, flood_img);
+	add_a_toggle(_("Gradient-driven"), box, flood_slide);
+	filter_window(_("Fill settings"), box, set_flood, NULL, TRUE);
+}
+
 static int set_smudge(GtkWidget *box, gpointer fdata)
 {
 	GtkWidget *toggle;
@@ -390,6 +248,14 @@ static int set_smudge(GtkWidget *box, gpointer fdata)
 	return TRUE;
 }
 
+void smudge_settings() /* Smudge opacity mode */
+{
+	GtkWidget *box = gtk_vbox_new(FALSE, 5);
+	gtk_widget_show(box);
+	add_a_toggle(_("Respect opacity mode"), box, smudge_mode);
+	filter_window(_("Smudge settings"), box, set_smudge, NULL, TRUE);
+}
+
 static int set_brush_step(GtkWidget *box, gpointer fdata)
 {
 	GtkWidget *spin;
@@ -400,11 +266,21 @@ static int set_brush_step(GtkWidget *box, gpointer fdata)
 	return TRUE;
 }
 
-static int blendtemp[5];
+void step_settings() /* Brush spacing */
+{
+	GtkWidget *box = gtk_vbox_new(FALSE, 5);
+	gtk_widget_show(box);
+	pack(box, add_a_spin(brush_spacing, 0, MAX_WIDTH));
+// !!! Not implemented yet
+//	add_a_toggle(_("Flat gradient strokes"), box, ???);
+	filter_window(_("Brush spacing"), box, set_brush_step, NULL, TRUE);
+}
+
+#define BLENDTEMP_SIZE 5
 
 static int set_blend(GtkWidget *box, gpointer fdata)
 {
-	int i, j;
+	int i, j, *blendtemp = fdata;
 
 	i = blendtemp[0] < 0 ? BLEND_NORMAL : blendtemp[0];
 	j = !blendtemp[2] + (!blendtemp[3] << 1) + (!blendtemp[4] << 2);
@@ -412,86 +288,41 @@ static int set_blend(GtkWidget *box, gpointer fdata)
 	/* Don't accept stop-all or do-nothing */
 	if ((j == 7) || !(i | j)) return (FALSE);
 
-	blend_mode = i | (blendtemp[1] ? BLEND_REVERSE : 0) |
-		(j << BLEND_RGBSHIFT);
+	blend_mode = i | (blendtemp[1] ? BLEND_REVERSE : 0) | (j << BLEND_RGBSHIFT);
 
 	return (TRUE);
 }
 
-static gboolean toolbar_rclick(GtkWidget *widget, GdkEventButton *event,
-	gpointer user_data)
+void blend_settings() /* Blend mode */
 {
 	char *rgbnames[3] = { _("Red"), _("Green"), _("Blue") };
 	char *blends[BLEND_NMODES] = {
 		_("Normal"), _("Hue"), _("Saturation"), _("Value"),
-		_("Colour"), _("Saturate More") };
+		_("Colour"), _("Saturate More"),
+		_("Multiply"), _("Divide"), _("Screen"), _("Dodge"),
+		_("Burn"), _("Hard Light"), _("Soft Light"), _("Difference"),
+		_("Darken"), _("Lighten"), _("Grain Extract"),
+		_("Grain Merge") };
 	GtkWidget *box, *hbox;
-	int i, k;
+	int i, *blendtemp;
 
-	/* Handle only right clicks */
-	if ((event->type != GDK_BUTTON_PRESS) || (event->button != 3))
-		return (FALSE);
-
-	switch ((gint)user_data)
+	box = gtk_vbox_new(FALSE, 5);
+	blendtemp = bound_malloc(box, BLENDTEMP_SIZE * sizeof(int));
+	gtk_container_set_border_width(GTK_CONTAINER(box), 5);
+	pack(box, wj_combo_box(blends, BLEND_NMODES,
+		blend_mode & BLEND_MMASK, blendtemp + 0, NULL));
+	pack(box, sig_toggle(_("Reverse"), blend_mode & BLEND_REVERSE,
+		blendtemp + 1, NULL));
+	add_hseparator(box, -2, 10);
+	hbox = pack(box, gtk_hbox_new(TRUE, 5));
+	for (i = 0; i < 3; i++)
 	{
-	case SETB_CONT: /* Brush spacing */
-		box = gtk_vbox_new(FALSE, 5);
-		gtk_widget_show(box);
-		pack(box, add_a_spin(brush_spacing, 0, MAX_WIDTH));
-// !!! Not implemented yet
-//		add_a_toggle(_("Flat gradient strokes"), box, ???);
-		filter_window(_("Brush spacing"), box, set_brush_step, NULL, TRUE);
-		break;
-	case SETB_CSEL:
-		colour_selector(COLSEL_EDIT_CSEL);
-		break;
-	case SETB_GRAD: /* Gradient selector */
-		gradient_setup(1);
-		break;
-	case SETB_FILT: /* Blend mode */
-		box = gtk_vbox_new(FALSE, 5);
-		gtk_container_set_border_width(GTK_CONTAINER(box), 5);
-		k = 0;
-		pack(box, wj_combo_box(blends, BLEND_NMODES,
-			blend_mode & BLEND_MMASK, blendtemp + 0, NULL));
-		pack(box, sig_toggle(_("Reverse"), blend_mode & BLEND_REVERSE,
-			blendtemp + 1, NULL));
-		add_hseparator(box, -2, 10);
-		hbox = pack(box, gtk_hbox_new(TRUE, 5));
-		for (i = 0; i < 3; i++)
-		{
-			pack(hbox, sig_toggle(rgbnames[i],
-				~blend_mode & ((1 << BLEND_RGBSHIFT) << i),
-				blendtemp + 2 + i, NULL));
-		}
-		gtk_widget_show_all(box);
-		filter_window(_("Blend mode"), box, set_blend, NULL, TRUE);
-		break;
-	case (TTB_0 + TTB_FLOOD): /* Flood fill step */
-		box = gtk_vbox_new(FALSE, 5);
-		gtk_widget_show(box);
-		pack(box, add_float_spin(flood_step, 0, 200));
-		add_a_toggle(_("RGB Cube"), box, flood_cube);
-		add_a_toggle(_("By image channel"), box, flood_img);
-		add_a_toggle(_("Gradient-driven"), box, flood_slide);
-		filter_window(_("Fill settings"), box, set_flood, NULL, TRUE);
-		break;
-	case (TTB_0 + TTB_SMUDGE): /* Smudge opacity mode */
-		box = gtk_vbox_new(FALSE, 5);
-		gtk_widget_show(box);
-		add_a_toggle(_("Respect opacity mode"), box, smudge_mode);
-		filter_window(_("Smudge settings"), box, set_smudge, NULL, TRUE);
-		break;
-	case (TTB_0 + TTB_GRAD): /* Gradient config */
-		gradient_setup(0);
-		break;
-	case (TTB_0 + TTB_TEXT): /* FreeType text paste */
-		pressed_mt_text(NULL, NULL);
-		break;
-	default: /* For other buttons, do nothing */
-		return (FALSE);
+		pack(hbox, sig_toggle(rgbnames[i],
+			~blend_mode & ((1 << BLEND_RGBSHIFT) << i),
+			blendtemp + 2 + i, NULL));
 	}
-	return (TRUE);
+	gtk_widget_show_all(box);
+	filter_window(_("Blend mode"), box, set_blend, (gpointer)blendtemp, TRUE);
 }
 
 
@@ -499,25 +330,25 @@ static void ts_update_spinslides()
 {
 	mt_spinslide_set_value(ts_spinslides[0], tool_size);
 	mt_spinslide_set_value(ts_spinslides[1], tool_flow);
-	mt_spinslide_set_value(ts_spinslides[2], mem_channel == CHN_IMAGE ?
-		tool_opacity : channel_col_A[mem_channel]);
+	mt_spinslide_set_value(ts_spinslides[2], tool_opacity);
+	if (mem_channel != CHN_IMAGE)
+		mt_spinslide_set_value(ts_spinslides[3], channel_col_A[mem_channel]);
 }
 
 
 static void ts_spinslide_moved(GtkAdjustment *adj, gpointer user_data)
 {
-	int n, i;
+	int n = ADJ2INT(adj);
 
-	n = ADJ2INT(adj);
 	switch ((int)user_data)
 	{
 	case 0: tool_size = n;
 		break;
 	case 1:	tool_flow = n;
 		break;
-	case 2:	i = mem_channel == CHN_IMAGE ? tool_opacity :
-			channel_col_A[mem_channel];
-		if (n != i) pressed_opacity(n);
+	case 2:	if (n != tool_opacity) pressed_opacity(n);
+		break;
+	case 3: if (n != channel_col_A[mem_channel]) pressed_value(n);
 		break;
 	}
 }
@@ -531,32 +362,54 @@ static void toolbar_settings_exit()
 	toolbar_exit();
 }
 
-static void fill_toolbar(GtkToolbar *bar, toolbar_item *items,
-	GtkSignalFunc lclick, int lbase, GtkSignalFunc rclick, int rbase)
+static void toolbar_click(GtkWidget *widget, gpointer user_data)
 {
-	GtkWidget *iconw, *radio[32];
-	GdkPixmap *icon, *mask;
+	toolbar_item *item = user_data;
+
+	action_dispatch(item->action, item->mode, item->radio < 0 ? TRUE :
+		GTK_TOGGLE_BUTTON(widget)->active, FALSE);
+}
+
+static gboolean toolbar_rclick(GtkWidget *widget, GdkEventButton *event,
+	gpointer user_data)
+{
+	toolbar_item *item = user_data;
+
+	/* Handle only right clicks */
+	if ((event->type != GDK_BUTTON_PRESS) || (event->button != 3))
+		return (FALSE);
+	action_dispatch(item->action2, item->mode2, TRUE, FALSE);
+	return (TRUE);
+}
+
+void fill_toolbar(GtkToolbar *bar, toolbar_item *items, GtkWidget **wlist,
+	GtkSignalFunc lclick, GtkSignalFunc rclick)
+{
+	GtkWidget *item, *radio[32];
+
+	if (!lclick) lclick = GTK_SIGNAL_FUNC(toolbar_click);
+	if (!rclick) rclick = GTK_SIGNAL_FUNC(toolbar_rclick);
 
 	memset(radio, 0, sizeof(radio));
-	for (; items->xpm; items++)
+	for (; items->tooltip; items++)
 	{
-		icon = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
-			NULL, items->xpm);
-		iconw = gtk_pixmap_new(icon, mask);
-		gdk_pixmap_unref(icon);
-		gdk_pixmap_unref(mask);
-		items->widget = gtk_toolbar_append_element(bar,
+		if (!items->xpm) // This is a separator
+		{
+			gtk_toolbar_append_space(bar);
+			continue;
+		}
+		item = gtk_toolbar_append_element(bar,
 			items->radio < 0 ? GTK_TOOLBAR_CHILD_BUTTON :
 			items->radio ? GTK_TOOLBAR_CHILD_RADIOBUTTON :
 			GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
 			items->radio > 0 ? radio[items->radio] : NULL,
-			NULL, _(items->tooltip), "Private", iconw, lclick,
-			(gpointer)(items->ID + lbase));
-		if (items->radio > 0) radio[items->radio] = items->widget;
-		if (items->rclick) gtk_signal_connect(GTK_OBJECT(items->widget),
-			"button_press_event", rclick, (gpointer)(items->ID + rbase));
-		if (items->sep) gtk_toolbar_append_space(bar);
-		mapped_dis_add(items->widget, items->actmap);
+			NULL, _(items->tooltip), "Private",
+			xpm_image(items->xpm), lclick, (gpointer)items);
+		if (items->radio > 0) radio[items->radio] = item;
+		if (items->action2) gtk_signal_connect(GTK_OBJECT(item),
+			"button_press_event", rclick, (gpointer)items);
+		mapped_dis_add(item, items->actmap);
+		if (wlist) wlist[items->ID] = item;
 	}
 }
 
@@ -568,26 +421,28 @@ static GtkWidget *grad_view;
 #define _(X) X
 
 static toolbar_item settings_bar[] = {
-	{ SETB_CONT, 0, 0, 1, 0, _("Continuous Mode"), xpm_mode_cont_xpm },
-	{ SETB_OPAC, 0, 0, 0, 0, _("Opacity Mode"), xpm_mode_opac_xpm },
-	{ SETB_TINT, 0, 0, 0, 0, _("Tint Mode"), xpm_mode_tint_xpm },
-	{ SETB_TSUB, 0, 0, 0, 0, _("Tint +-"), xpm_mode_tint2_xpm },
-	{ SETB_CSEL, 0, 0, 1, 0, _("Colour-Selective Mode"), xpm_mode_csel_xpm },
-	{ SETB_FILT, 0, 0, 1, 0, _("Blend Mode"), xpm_mode_blend_xpm },
-	{ SETB_MASK, 0, 0, 0, 0, _("Disable All Masks"), xpm_mode_mask_xpm },
-	{ 0, 0, 0, 0, 0, NULL, NULL }};
+	{ _("Continuous Mode"), 0, SETB_CONT, 0, xpm_mode_cont_xpm, ACT_MODE, SETB_CONT, DLG_STEP, 0 },
+	{ _("Opacity Mode"), 0, SETB_OPAC, 0, xpm_mode_opac_xpm, ACT_MODE, SETB_OPAC },
+	{ _("Tint Mode"), 0, SETB_TINT, 0, xpm_mode_tint_xpm, ACT_MODE, SETB_TINT },
+	{ _("Tint +-"), 0, SETB_TSUB, 0, xpm_mode_tint2_xpm, ACT_MODE, SETB_TSUB },
+	{ _("Colour-Selective Mode"), 0, SETB_CSEL, 0, xpm_mode_csel_xpm, ACT_MODE, SETB_CSEL, DLG_COLORS, COLSEL_EDIT_CSEL },
+	{ _("Blend Mode"), 0, SETB_FILT, 0, xpm_mode_blend_xpm, ACT_MODE, SETB_FILT, DLG_FILT, 0 },
+	{ _("Disable All Masks"), 0, SETB_MASK, 0, xpm_mode_mask_xpm, ACT_MODE, SETB_MASK },
+	{ NULL }};
+
+static toolbar_item gradient_button =
+	{ _("Gradient Mode"), 0, SETB_GRAD, 0, NULL, ACT_MODE, SETB_GRAD, DLG_GRAD, 1 };
 
 #undef _
 #define _(X) __(X)
 
 static void toolbar_settings_init()
 {
-	int i, vals[] = {tool_size, tool_flow, tool_opacity};
-	char *ts_titles[] = { _("Size"), _("Flow"), _("Opacity") };
-
-	GtkWidget *label, *vbox, *table, *toolbar_settings, *labels[3];
+	char *ts_titles[4] = { _("Size"), _("Flow"), _("Opacity"), "" };
+	GtkWidget *label, *vbox, *table, *toolbar_settings;
 	GtkWidget *button;
 	GdkPixmap *pmap;
+	int i;
 
 
 	if ( toolbar_boxes[TOOLBAR_SETTINGS] )
@@ -622,19 +477,11 @@ static void toolbar_settings_init()
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar_settings), GTK_TOOLBAR_ICONS);
 #endif
 
-	fill_toolbar(GTK_TOOLBAR(toolbar_settings), settings_bar,
-		GTK_SIGNAL_FUNC(toolbar_mode_change), 0,
-		GTK_SIGNAL_FUNC(toolbar_rclick), 0);
-
-	for (i = 0; i < TOTAL_ICONS_SETTINGS; i++)
-	{
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(settings_bar[i].widget),
-			!!*(vars_settings[i]));
-	}
-
+	fill_toolbar(GTK_TOOLBAR(toolbar_settings), settings_bar, settings_buttons, NULL, NULL);
 	gtk_widget_show(toolbar_settings);
 
 	/* Gradient mode button+preview */
+	settings_buttons[SETB_GRAD] = button = pack(vbox, gtk_toggle_button_new());
 	button = pack(vbox, gtk_toggle_button_new());
 	pmap = gdk_pixmap_new(main_window->window, GP_WIDTH, GP_HEIGHT, -1);
 	grad_view = gtk_pixmap_new(pmap, NULL);
@@ -644,16 +491,21 @@ static void toolbar_settings_init()
 #if (GTK_MAJOR_VERSION == 2) && (GTK_MINOR_VERSION >= 4) /* GTK+ 2.4+ */
 	gtk_button_set_focus_on_click(GTK_BUTTON(button), FALSE);
 #endif
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), !!mem_gradient);
 	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		GTK_SIGNAL_FUNC(toolbar_mode_change), (gpointer)SETB_GRAD);
+		GTK_SIGNAL_FUNC(toolbar_click), (gpointer)&gradient_button);
 	gtk_signal_connect(GTK_OBJECT(button), "button_press_event",
-		GTK_SIGNAL_FUNC(toolbar_rclick), (gpointer)SETB_GRAD);
+		GTK_SIGNAL_FUNC(toolbar_rclick), (gpointer)&gradient_button);
 	gtk_widget_show_all(button);
 	gtk_widget_realize(grad_view);
 	/* Parasite gradient tooltip on settings toolbar */
 	gtk_tooltips_set_tip(GTK_TOOLBAR(toolbar_settings)->tooltips,
-		button, _("Gradient Mode"), "Private");
+		button, gradient_button.tooltip, "Private");
+
+	for (i = 0; i < TOTAL_SETTINGS; i++) // Initialize buttons' state
+	{
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(settings_buttons[i]),
+			!!*(vars_settings[i]));
+	}
 
 	/* Colors A & B */
 	label = pack(vbox, gtk_label_new(""));
@@ -668,19 +520,21 @@ static void toolbar_settings_init()
 	gtk_widget_show (label);
 	gtk_misc_set_padding (GTK_MISC (label), 5, 2);
 
-	if (mem_channel != CHN_IMAGE) vals[2] = channel_col_A[mem_channel];
-	table = add_a_table(3, 2, 5, vbox);
-	for (i = 0; i < 3; i++)
+	table = pack_end(vbox, gtk_table_new(4, 2, FALSE));
+	gtk_widget_show(table);
+	gtk_container_set_border_width(GTK_CONTAINER(table), 5);
+	for (i = 0; i < 4; i++)
 	{
-		labels[i] = add_to_table(ts_titles[i], table, i, 0, 0);
+		label = add_to_table(ts_titles[i], table, i, 0, 0);
 		ts_spinslides[i] = mt_spinslide_new(-1, -1);
 		gtk_table_attach(GTK_TABLE(table), ts_spinslides[i], 1, 2,
 			i, i + 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-		mt_spinslide_set_range(ts_spinslides[i], i == 2 ? 0 : 1, 255);
+		mt_spinslide_set_range(ts_spinslides[i], i < 2 ? 1 : 0, 255);
 		mt_spinslide_connect(ts_spinslides[i],
 			GTK_SIGNAL_FUNC(ts_spinslide_moved), (gpointer)i);
 	}
-	tb_label_opacity = labels[2];
+// !!! Use the fact that channel value slider is the last one
+	ts_label_channel = label;
 
 	gtk_signal_connect(GTK_OBJECT(toolbar_boxes[TOOLBAR_SETTINGS]),
 		"delete_event", GTK_SIGNAL_FUNC(toolbar_settings_exit), NULL);
@@ -696,52 +550,51 @@ static void toolbar_settings_init()
 #define _(X) X
 
 static toolbar_item main_bar[] = {
-	{ MTB_NEW, -1, 0, 0, 0, _("New Image"), xpm_new_xpm },
-	{ MTB_OPEN, -1, 0, 0, 0, _("Load Image File"), xpm_open_xpm },
-	{ MTB_SAVE, -1, 1, 0, 0, _("Save Image File"), xpm_save_xpm },
-	{ MTB_CUT, -1, 0, 0, NEED_SEL2, _("Cut"), xpm_cut_xpm },
-	{ MTB_COPY, -1, 0, 0, NEED_SEL2, _("Copy"), xpm_copy_xpm },
-	{ MTB_PASTE, -1, 1, 0, NEED_CLIP, _("Paste"), xpm_paste_xpm },
-	{ MTB_UNDO, -1, 0, 0, NEED_UNDO, _("Undo"), xpm_undo_xpm },
-	{ MTB_REDO, -1, 1, 0, NEED_REDO, _("Redo"), xpm_redo_xpm },
-	{ MTB_BRCOSA, -1, 0, 0, 0, _("Transform Colour"), xpm_brcosa_xpm },
-	{ MTB_PAN, -1, 0, 0, 0, _("Pan Window"), xpm_pan_xpm },
-	{ 0, 0, 0, 0, 0, NULL, NULL }};
+	{ _("New Image"), -1, MTB_NEW, 0, xpm_new_xpm, DLG_NEW, 0 },
+	{ _("Load Image File"), -1, MTB_OPEN, 0, xpm_open_xpm, DLG_FSEL, FS_PNG_LOAD },
+	{ _("Save Image File"), -1, MTB_SAVE, 0, xpm_save_xpm, ACT_SAVE, 0 },
+	{""},
+	{ _("Cut"), -1, MTB_CUT, NEED_SEL2, xpm_cut_xpm, ACT_COPY, 1 },
+	{ _("Copy"), -1, MTB_COPY, NEED_SEL2, xpm_copy_xpm, ACT_COPY, 0 },
+	{ _("Paste"), -1, MTB_PASTE, NEED_CLIP, xpm_paste_xpm, ACT_PASTE, 0 },
+	{""},
+	{ _("Undo"), -1, MTB_UNDO, NEED_UNDO, xpm_undo_xpm, ACT_UNDO, 0 },
+	{ _("Redo"), -1, MTB_REDO, NEED_REDO, xpm_redo_xpm, ACT_REDO, 0 },
+	{""},
+	{ _("Transform Colour"), -1, MTB_BRCOSA, 0, xpm_brcosa_xpm, DLG_BRCOSA, 0 },
+	{ _("Pan Window"), -1, MTB_PAN, 0, xpm_pan_xpm, ACT_PAN, 0 },
+	{ NULL }};
 
 static toolbar_item tools_bar[] = {
-	{ TTB_PAINT, 1, 0, 0, 0, _("Paint"), xpm_paint_xpm },
-	{ TTB_SHUFFLE, 1, 0, 0, 0, _("Shuffle"), xpm_shuffle_xpm },
-	{ TTB_FLOOD, 1, 0, 1, 0, _("Flood Fill"), xpm_flood_xpm },
-	{ TTB_LINE, 1, 0, 0, 0, _("Straight Line"), xpm_line_xpm },
-	{ TTB_SMUDGE, 1, 0, 1, NEED_24, _("Smudge"), xpm_smudge_xpm },
-	{ TTB_CLONE, 1, 0, 0, 0, _("Clone"), xpm_clone_xpm },
-	{ TTB_SELECT, 1, 0, 0, 0, _("Make Selection"), xpm_select_xpm },
-	{ TTB_POLY, 1, 0, 0, 0, _("Polygon Selection"), xpm_polygon_xpm },
-	{ TTB_GRAD, 1, 1, 1, 0, _("Place Gradient"), xpm_grad_place_xpm },
-	{ TTB_LASSO, -1, 0, 0, NEED_LASSO, _("Lasso Selection"), xpm_lasso_xpm },
-	{ TTB_TEXT, -1, 1, 1, 0, _("Paste Text"), xpm_text_xpm },
-	{ TTB_ELLIPSE, -1, 0, 0, NEED_SEL, _("Ellipse Outline"), xpm_ellipse2_xpm },
-	{ TTB_FELLIPSE, -1, 0, 0, NEED_SEL, _("Filled Ellipse"), xpm_ellipse_xpm },
-	{ TTB_OUTLINE, -1, 0, 0, NEED_SEL2, _("Outline Selection"), xpm_rect1_xpm },
-	{ TTB_FILL, -1, 1, 0, NEED_SEL2, _("Fill Selection"), xpm_rect2_xpm },
-	{ TTB_SELFV, -1, 0, 0, NEED_CLIP, _("Flip Selection Vertically"), xpm_flip_vs_xpm },
-	{ TTB_SELFH, -1, 0, 0, NEED_CLIP, _("Flip Selection Horizontally"), xpm_flip_hs_xpm },
-	{ TTB_SELRCW, -1, 0, 0, NEED_CLIP, _("Rotate Selection Clockwise"), xpm_rotate_cs_xpm },
-	{ TTB_SELRCCW, -1, 0, 0, NEED_CLIP, _("Rotate Selection Anti-Clockwise"), xpm_rotate_as_xpm },
-	{ 0, 0, 0, 0, 0, NULL, NULL }};
+	{ _("Paint"), 1, TTB_PAINT, 0, xpm_paint_xpm, ACT_TOOL, TTB_PAINT },
+	{ _("Shuffle"), 1, TTB_SHUFFLE, 0, xpm_shuffle_xpm, ACT_TOOL, TTB_SHUFFLE },
+	{ _("Flood Fill"), 1, TTB_FLOOD, 0, xpm_flood_xpm, ACT_TOOL, TTB_FLOOD, DLG_FLOOD, 0 },
+	{ _("Straight Line"), 1, TTB_LINE, 0, xpm_line_xpm, ACT_TOOL, TTB_LINE },
+	{ _("Smudge"), 1, TTB_SMUDGE, NEED_24, xpm_smudge_xpm, ACT_TOOL, TTB_SMUDGE, DLG_SMUDGE, 0 },
+	{ _("Clone"), 1, TTB_CLONE, 0, xpm_clone_xpm, ACT_TOOL, TTB_CLONE },
+	{ _("Make Selection"), 1, TTB_SELECT, 0, xpm_select_xpm, ACT_TOOL, TTB_SELECT },
+	{ _("Polygon Selection"), 1, TTB_POLY, 0, xpm_polygon_xpm, ACT_TOOL, TTB_POLY },
+	{ _("Place Gradient"), 1, TTB_GRAD, 0, xpm_grad_place_xpm, ACT_TOOL, TTB_GRAD, DLG_GRAD, 0 },
+	{""},
+	{ _("Lasso Selection"), -1, TTB_LASSO, NEED_LAS2, xpm_lasso_xpm, ACT_LASSO, 0 },
+	{ _("Paste Text"), -1, TTB_TEXT, 0, xpm_text_xpm, DLG_TEXT, 0, DLG_TEXT_FT, 0 },
+	{""},
+	{ _("Ellipse Outline"), -1, TTB_ELLIPSE, NEED_SEL, xpm_ellipse2_xpm, ACT_ELLIPSE, 0 },
+	{ _("Filled Ellipse"), -1, TTB_FELLIPSE, NEED_SEL, xpm_ellipse_xpm, ACT_ELLIPSE, 1 },
+	{ _("Outline Selection"), -1, TTB_OUTLINE, NEED_SEL2, xpm_rect1_xpm, ACT_OUTLINE, 0 },
+	{ _("Fill Selection"), -1, TTB_FILL, NEED_SEL2, xpm_rect2_xpm, ACT_OUTLINE, 1 },
+	{""},
+	{ _("Flip Selection Vertically"), -1, TTB_SELFV, NEED_CLIP, xpm_flip_vs_xpm, ACT_SEL_FLIP_V, 0 },
+	{ _("Flip Selection Horizontally"), -1, TTB_SELFH, NEED_CLIP, xpm_flip_hs_xpm, ACT_SEL_FLIP_H, 0 },
+	{ _("Rotate Selection Clockwise"), -1, TTB_SELRCW, NEED_CLIP, xpm_rotate_cs_xpm, ACT_SEL_ROT, 0 },
+	{ _("Rotate Selection Anti-Clockwise"), -1, TTB_SELRCCW, NEED_CLIP, xpm_rotate_as_xpm, ACT_SEL_ROT, 1 },
+	{ NULL }};
 
 #undef _
 #define _(X) __(X)
 
 void toolbar_init(GtkWidget *vbox_main)
 {
-	char txt[32];
-	int i;
-
-	GdkPixmap *icon, *mask;
-	GtkWidget *toolbar_main, *toolbar_tools, *hbox;
-	GdkColor cfg = { -1, -1, -1, -1 }, cbg = { 0, 0, 0, 0 };
-
 	static char *xbm_list[TOTAL_CURSORS] = { xbm_square_bits, xbm_circle_bits,
 		xbm_horizontal_bits, xbm_vertical_bits, xbm_slash_bits, xbm_backslash_bits,
 		xbm_spray_bits, xbm_shuffle_bits, xbm_flood_bits, xbm_select_bits, xbm_line_bits,
@@ -757,6 +610,15 @@ void toolbar_init(GtkWidget *vbox_main)
 	static unsigned char cursor_tip[TOTAL_CURSORS][2] = { {0, 0}, {0, 0},
 		{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {2, 19},
 		{10, 10}, {0, 0}, {0, 0}, {10, 10}, {0, 0}, {0, 0} };
+	GtkWidget *toolbar_main, *toolbar_tools, *box;
+	char txt[32];
+	int i;
+
+
+	for (i=0; i<TOTAL_CURSORS; i++)
+		m_cursor[i] = make_cursor(xbm_list[i], xbm_mask_list[i],
+			20, 20, cursor_tip[i][0], cursor_tip[i][1]);
+	move_cursor = make_cursor(xbm_move_bits, xbm_move_mask_bits, 20, 20, 10, 10);
 
 	for ( i=1; i<TOOLBAR_MAX; i++ )
 	{
@@ -770,20 +632,21 @@ void toolbar_init(GtkWidget *vbox_main)
 
 ///	MAIN TOOLBAR
 
-	toolbar_boxes[TOOLBAR_MAIN] = hbox = pack(vbox_main, gtk_hbox_new(FALSE, 0));
-	if (toolbar_status[TOOLBAR_MAIN]) gtk_widget_show(hbox); // Only show if user wants
+	toolbar_boxes[TOOLBAR_MAIN] = box = pack(vbox_main, gtk_alignment_new(0, 0, 0, 1));
+	if (toolbar_status[TOOLBAR_MAIN]) gtk_widget_show(box); // Only show if user wants
 
 #if GTK_MAJOR_VERSION == 1
-	toolbar_main = pack(hbox, gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,
-		GTK_TOOLBAR_ICONS));
-#endif
-#if GTK_MAJOR_VERSION == 2
-	toolbar_main = pack(hbox, gtk_toolbar_new());
-	gtk_toolbar_set_style( GTK_TOOLBAR(toolbar_main), GTK_TOOLBAR_ICONS );
+	toolbar_main = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
+#else /* #if GTK_MAJOR_VERSION == 2 */
+	toolbar_main = gtk_toolbar_new();
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar_main), GTK_TOOLBAR_ICONS);
 #endif
+	gtk_container_add(GTK_CONTAINER(box), toolbar_main);
 
-	toolbar_zoom_main = toolbar_add_zoom(hbox);
-	toolbar_zoom_view = toolbar_add_zoom(hbox);
+	fill_toolbar(GTK_TOOLBAR(toolbar_main), main_bar, NULL, NULL, NULL);
+
+	toolbar_zoom_main = toolbar_add_zoom(toolbar_main);
+	toolbar_zoom_view = toolbar_add_zoom(toolbar_main);
 	/* In GTK1, combo box entry is updated continuously */
 #if GTK_MAJOR_VERSION == 1
 	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(toolbar_zoom_main)->popwin),
@@ -799,51 +662,25 @@ void toolbar_init(GtkWidget *vbox_main)
 #endif
 	toolbar_viewzoom(FALSE);
 
-	for (i=0; i<TOTAL_CURSORS; i++)
-	{
-		icon = gdk_bitmap_create_from_data (NULL, xbm_list[i], 20, 20);
-		mask = gdk_bitmap_create_from_data (NULL, xbm_mask_list[i], 20, 20);
-
-		m_cursor[i] = gdk_cursor_new_from_pixmap(icon, mask, &cfg, &cbg,
-			cursor_tip[i][0], cursor_tip[i][1]);
-
-		gdk_pixmap_unref( icon );
-		gdk_pixmap_unref( mask );
-	}
-	icon = gdk_bitmap_create_from_data (NULL, xbm_move_bits, 20, 20);
-	mask = gdk_bitmap_create_from_data (NULL, xbm_move_mask_bits, 20, 20);
-	move_cursor = gdk_cursor_new_from_pixmap (icon, mask, &cfg, &cbg, 10, 10);
-	gdk_pixmap_unref( icon );
-	gdk_pixmap_unref( mask );
-
-	fill_toolbar(GTK_TOOLBAR(toolbar_main), main_bar,
-		GTK_SIGNAL_FUNC(toolbar_icon_event2), 0, NULL, 0);
-
 	gtk_widget_show(toolbar_main);
 
 
 ///	TOOLS TOOLBAR
 
-	toolbar_boxes[TOOLBAR_TOOLS] = hbox = pack(vbox_main, gtk_hbox_new(FALSE, 0));
-	if (toolbar_status[TOOLBAR_TOOLS]) gtk_widget_show(hbox); // Only show if user wants
+	toolbar_boxes[TOOLBAR_TOOLS] = box = pack(vbox_main, gtk_alignment_new(0, 0, 0, 1));
+	if (toolbar_status[TOOLBAR_TOOLS]) gtk_widget_show(box); // Only show if user wants
 
 #if GTK_MAJOR_VERSION == 1
-	toolbar_tools = pack(hbox, gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,
-		GTK_TOOLBAR_ICONS));
-#endif
-#if GTK_MAJOR_VERSION == 2
-	toolbar_tools = pack(hbox, gtk_toolbar_new());
-	gtk_toolbar_set_style( GTK_TOOLBAR(toolbar_tools), GTK_TOOLBAR_ICONS );
+	toolbar_tools = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
+#else /* #if GTK_MAJOR_VERSION == 2 */
+	toolbar_tools = gtk_toolbar_new();
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar_tools), GTK_TOOLBAR_ICONS);
 #endif
+	gtk_container_add(GTK_CONTAINER(box), toolbar_tools);
 
-	fill_toolbar(GTK_TOOLBAR(toolbar_tools), tools_bar,
-		GTK_SIGNAL_FUNC(toolbar_icon_event), 0,
-		GTK_SIGNAL_FUNC(toolbar_rclick), TTB_0);
-	for (i = 0; tools_bar[i].xpm; i++)
-	{
-		icon_buttons[tools_bar[i].ID] = tools_bar[i].widget;
-	}
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(icon_buttons[PAINT_TOOL_ICON]), TRUE);
+	fill_toolbar(GTK_TOOLBAR(toolbar_tools), tools_bar, icon_buttons, NULL, NULL);
+	/* !!! If hardcoded default tool isn't the default brush, this will crash */
+	change_to_tool(TTB_PAINT);
 
 	gtk_widget_show(toolbar_tools);
 }
@@ -879,8 +716,7 @@ void ts_update_gradient()
 		}
 	}
 	else tmp = cset , --slot; /* Use gradient colors */
-	if ((mem_img_bpp == 3) && (mem_channel <= CHN_ALPHA))
-		idx = 0; /* Allow intermediate opacities */
+	if (!IS_INDEXED) idx = 0; /* Allow intermediate opacities */
 
 	/* Draw the preview, ignoring RGBA coupling */
 	memset(rgb, mem_background, sizeof(rgb));
@@ -928,30 +764,40 @@ void ts_update_gradient()
 void toolbar_update_settings()
 {
 	char txt[32];
+	int i, j, c;
 
 	if ( toolbar_boxes[TOOLBAR_SETTINGS] == NULL ) return;
 
+	for (i = 0; i < 2; i++)
+	{
+		c = "AB"[i]; j = mem_col_[i];
+		if (mem_img_bpp == 1) snprintf(txt, 30, "%c [%d] = {%d,%d,%d}",
+			c, j, mem_pal[j].red, mem_pal[j].green, mem_pal[j].blue);
+		else snprintf(txt, 30, "%c = {%i,%i,%i}", c, mem_col_24[i].red,
+			mem_col_24[i].green, mem_col_24[i].blue);
+		gtk_label_set_text(GTK_LABEL(toolbar_labels[i]), txt);
+	}
+
+	if (mem_channel == CHN_IMAGE)
+	{
+		gtk_widget_show(toolbar_labels[0]);
+		gtk_widget_show(toolbar_labels[1]);
+		gtk_widget_hide(ts_label_channel);
+		gtk_widget_hide(ts_spinslides[3]);
+	}
+	else
+	{
+		gtk_label_set_text(GTK_LABEL(ts_label_channel), channames[mem_channel]);
+		gtk_widget_hide(toolbar_labels[0]);
+		gtk_widget_hide(toolbar_labels[1]);
+		gtk_widget_show(ts_label_channel);
+		gtk_widget_show(ts_spinslides[3]);
+	}
+	// Disable opacity for indexed image
+	gtk_widget_set_sensitive(ts_spinslides[2], !IS_INDEXED);
+
 	ts_update_spinslides();		// Update tool settings
 	ts_update_gradient();
-
-	if ( mem_img_bpp == 1 )
-		snprintf(txt, 30, "A [%i] = {%i,%i,%i}", mem_col_A, mem_pal[mem_col_A].red,
-			mem_pal[mem_col_A].green, mem_pal[mem_col_A].blue );
-	else	snprintf(txt, 30, "A = {%i,%i,%i}", mem_col_A24.red,
-			mem_col_A24.green, mem_col_A24.blue );
-	gtk_label_set_text( GTK_LABEL(toolbar_labels[0]), txt );
-
-	if ( mem_img_bpp == 1 )
-		snprintf(txt, 30, "B [%i] = {%i,%i,%i}", mem_col_B, mem_pal[mem_col_B].red,
-			mem_pal[mem_col_B].green, mem_pal[mem_col_B].blue );
-	else	snprintf(txt, 30, "B = {%i,%i,%i}", mem_col_B24.red,
-			mem_col_B24.green, mem_col_B24.blue );
-	gtk_label_set_text( GTK_LABEL(toolbar_labels[1]), txt );
-
-	if ( mem_channel == CHN_IMAGE )
-		gtk_label_set_text( GTK_LABEL(tb_label_opacity), _("Opacity") );
-	else
-		gtk_label_set_text( GTK_LABEL(tb_label_opacity), channames[mem_channel] );
 }
 
 static gboolean expose_palette(GtkWidget *widget, GdkEventExpose *event,
@@ -1007,14 +853,14 @@ static gboolean motion_palette(GtkWidget *widget, GdkEventMotion *event, gpointe
 	if (drag_index && (drag_index_vals[1] != pindex))
 	{
 		mem_pal_index_move(drag_index_vals[1], pindex);
-		init_pal();
+		update_stuff(UPD_MVPAL);
 		drag_index_vals[1] = pindex;
 	}
 
 	return (TRUE);
 }
 
-static gint release_palette( GtkWidget *widget, GdkEventButton *event )
+static gboolean release_palette( GtkWidget *widget, GdkEventButton *event )
 {
 	if (drag_index)
 	{
@@ -1030,14 +876,14 @@ static gint release_palette( GtkWidget *widget, GdkEventButton *event )
 				mem_canvas_index_move( drag_index_vals[0], drag_index_vals[1] );
 
 			mem_undo_prepare();
-			canvas_undo_chores();
+			update_stuff(UPD_PAL | CF_MENU);
 		}
 	}
 
 	return FALSE;
 }
 
-static gint click_palette( GtkWidget *widget, GdkEventButton *event )
+static gboolean click_palette( GtkWidget *widget, GdkEventButton *event )
 {
 	int px, py, pindex;
 
@@ -1054,49 +900,27 @@ static gint click_palette( GtkWidget *widget, GdkEventButton *event )
 	if (px < PALETTE_SWATCH_X) colour_selector(COLSEL_EDIT_ALL + pindex);
 	else if (px < PALETTE_CROSS_X)		// Colour A or B changed
 	{
-		if (event->button == 1)
+		if ((event->button == 1) && (event->state & GDK_SHIFT_MASK))
 		{
-			if (event->state & GDK_CONTROL_MASK)
-			{
-				mem_col_B = pindex;
-				mem_col_B24 = mem_pal[mem_col_B];
-			}
-			else if (event->state & GDK_SHIFT_MASK)
-			{
-				mem_pal_copy(brcosa_pal, mem_pal);
-				drag_index = TRUE;
-				drag_index_vals[0] = pindex;
-				drag_index_vals[1] = pindex;
-				gdk_window_set_cursor(drawing_palette->window, move_cursor);
-			}
-			else
-			{
-				mem_col_A = pindex;
-				mem_col_A24 = mem_pal[mem_col_A];
-			}
+			mem_pal_copy(brcosa_pal, mem_pal);
+			drag_index = TRUE;
+			drag_index_vals[0] = drag_index_vals[1] = pindex;
+			gdk_window_set_cursor(drawing_palette->window, move_cursor);
 		}
-		else if (event->button == 3)
+		else if ((event->button == 1) || (event->button == 3))
 		{
-			mem_col_B = pindex;
-			mem_col_B24 = mem_pal[mem_col_B];
-		}
+			int ab = (event->button == 3) || (event->state & GDK_CONTROL_MASK);
 
-		init_pal();
+			mem_col_[ab] = pindex;
+			mem_col_24[ab] = mem_pal[pindex];
+			update_stuff(UPD_AB);
+		}
 	}
 	else /* if (px >= PALETTE_CROSS_X) */		// Mask changed
 	{
 		mem_prot_mask[pindex] ^= 255;
-
-		/* Update & redraw swatch */
-		repaint_swatch(pindex);
-		gtk_widget_queue_draw_area(widget,
-			PALETTE_CROSS_X, PALETTE_SWATCH_Y + PALETTE_SWATCH_H * pindex,
-			PALETTE_WIDTH - PALETTE_CROSS_X, PALETTE_SWATCH_H);
-
 		mem_mask_init();		// Prepare RGB masks
-		/* !!! Do the same for any other kind of preview */
-		if ((tool_type == TOOL_SELECT) && (marq_status >= MARQUEE_PASTE))
-			update_all_views();
+		update_stuff(UPD_CMASK);
 	}
 
 	return (TRUE);
@@ -1125,9 +949,9 @@ void toolbar_palette_init(GtkWidget *box)		// Set up the palette area
 
 	viewport_palette = gtk_viewport_new (NULL, NULL);
 	gtk_widget_show (viewport_palette);
-	gtk_viewport_set_shadow_type( GTK_VIEWPORT(viewport_palette), GTK_SHADOW_IN );
 	gtk_container_add (GTK_CONTAINER (viewport_palette), drawing_col_prev);
 	gtk_box_pack_start( GTK_BOX(hbox), viewport_palette, TRUE, FALSE, 0 );
+	fix_vport(viewport_palette);
 
 	scrolledwindow_palette = xpack(vbox, gtk_scrolled_window_new(NULL, NULL));
 	gtk_widget_show (scrolledwindow_palette);
@@ -1179,33 +1003,30 @@ void toolbar_exit()				// Remember toolbar settings on program exit
 
 void toolbar_showhide()				// Show/Hide all 4 toolbars
 {
-	int i, bar[] = { TOOLBAR_MAIN, TOOLBAR_TOOLS, TOOLBAR_PALETTE, TOOLBAR_STATUS };
+	static const unsigned char bar[4] =
+		{ TOOLBAR_MAIN, TOOLBAR_TOOLS, TOOLBAR_PALETTE, TOOLBAR_STATUS };
+	int i;
 
-	if ( toolbar_boxes[TOOLBAR_MAIN] == NULL ) return;		// Grubby hack to avoid segfault
+	if (!toolbar_boxes[TOOLBAR_MAIN]) return;	// Grubby hack to avoid segfault
 
-	for ( i=0; i<4; i++ )
+	for (i = 0; i < 4; i++)
 	{
-		if ( toolbar_status[ bar[i] ] )
-			gtk_widget_show( toolbar_boxes[ bar[i] ] );
-		else
-			gtk_widget_hide( toolbar_boxes[ bar[i] ] );
+		(toolbar_status[bar[i]] ? gtk_widget_show :
+			gtk_widget_hide)(toolbar_boxes[bar[i]]);
 	}
 
-	if ( !toolbar_status[TOOLBAR_SETTINGS] )
-	{
-		toolbar_exit();
-	}
+	if (!toolbar_status[TOOLBAR_SETTINGS]) toolbar_exit();
 	else
 	{
 		toolbar_settings_init();
-		gdk_window_raise( main_window->window );
+		gdk_window_raise(main_window->window);
 	}
 }
 
 
-void pressed_toolbar_toggle( GtkMenuItem *menu_item, gpointer user_data, gint item )
+void pressed_toolbar_toggle(int state, int which)
 {						// Menu toggle for toolbars
-	toolbar_status[item] = GTK_CHECK_MENU_ITEM(menu_item)->active;;
+	toolbar_status[which] = state;
 	toolbar_showhide();
 }
 
@@ -1218,7 +1039,7 @@ void mem_set_brush(int val)			// Set brush, update size/flow/preview
 {
 	int offset, i, j, k, o, o2;
 
-	tool_type = mem_brush_list[val][0];
+	brush_tool_type = mem_brush_list[val][0];
 	tool_size = mem_brush_list[val][1];
 	if ( mem_brush_list[val][2]>0 ) tool_flow = mem_brush_list[val][2];
 
@@ -1234,8 +1055,6 @@ void mem_set_brush(int val)			// Set brush, update size/flow/preview
 				mem_prev[o + 3*i + k] = mem_brushes[o2 + 3*i + k];
 		}
 	}
-
-	if ( drawing_col_prev ) gtk_widget_queue_draw( drawing_col_prev );
 }
 
 #include "graphics/xbm_patterns.xbm"
@@ -1328,10 +1147,6 @@ void mem_pat_update()			// Update indexed and then RGB pattern preview
 			mem_prev[k + 2] = mem_col_pat24[l + 2];
 		}
 	}
-
-	grad_def_update();
-	if ((tool_type == TOOL_GRADIENT) && grad_opacity)
-		gtk_widget_queue_draw(drawing_canvas);
 }
 
 void update_top_swatch()			// Update selected colours A & B
